from .. import api, entities as en
from ..enums import *
from ..error import MondayClientError
from ..models import MondayModel


class MondayClient():
    """monday.com client for login and board/item management

        Properties

            me : `moncli.entities.User`
                The client login user.

        Methods

            create_board : `moncli.entities.Board`
                Create a new board.
            get_boards : `list[moncli.entities.Board]`
                Get a collection of boards.
            get_board : `moncli.entities.Board`
                Get a board by unique identifier or name.
            get_board_by_id : `moncli.entities.Board`
                Get a board by unique identifier.
            get_board_by_name : `moncli.entities.Board`
                Get a board by name.
            archive_board : `moncli.entities.Board`
                Archive a board.
            get_assets : `list[monlci.entities.Asset]`
                Get a collection of assets by IDs.
            get_items : `list[moncli.entities.Item]`
                Get a collection of items.
            get_updates : `list[moncli.entities.Update]`
                Get a collection of updates.
            delete_update : `moncli.entities.Update`
                Delete an update.
            clear_item_updates: `moncli.entities.Item`
                Clear an item's updates.
            create_notification : `moncli.entities.Notification`
                Create a new notification.
            create_or_get_tag : `moncli.entities.Tag`
                Create a new tag or get it if it already exists.
            get_tags : `list[moncli.entities.Tag]`
                Get a collection of tags.
            get_users : `list[moncli.entities.User]`
                Get a collection of users.
            get_teams : `list[moncli.entities.Team]`
                Get a collection of teams.
            get_me : `moncli.entities.User`
                Get the connected user's information.
            get_account : `moncli.entities.Account`
                Get the connected user's account.
            create_workspace : `moncli.entities.Workspace`
                Create a new workspace.
            add_users_to_workspace: `moncli.entities.Workspace`
                Adds new users to a new workspace.
            delete_users_from_workspace: `moncli.entities.Workspace`
                Deletes  users from an existing workspace.
            add_teams_to_workspace: `moncli.entities.Workspace`
                Allows you to add teams to a workspace.
            delete_teams_from_workspace: `moncli.entities.Workspace`
                Allows you to remove teams to a workspace.
    """

    def __init__(self, **kwargs):    
        self.__me = None
        self.__creds = en.MondayClientCredentials(kwargs.pop('api_key', None))

    @property
    def me(self):
        """Retrieve login user"""
        if not self.__me:
            self.__me = self.get_me()
        return self.__me

    @property
    def api_key(self):
        """Get API Key V2"""
        return self.__creds.api_key_v2

    @api_key.setter
    def api_key(self, value):
        """Set API Key V2"""
        self.__creds.api_key_v2 = value

    def create_board(self, board_name: str, board_kind: BoardKind, *args, **kwargs):
        """Create a new board.

            Parameters

                board_name : `str`
                    The board's name.
                board_kind : `moncli.enums.BoardKind`
                    The board's kind (public / private / share)
                args : `tuple`
                    The list of board return fields.
                kwargs : `dict`
                    Optional keyword arguments for creating a board.

            Returns

                board : `moncli.entities.Board`
                    The newly created board.

            Return Fields

                activity_logs : `list[moncli.entities.ActivityLog]`
                    The board log events.
                board_folder_id : `int`
                    The board's folder unique identifier.
                board_kind : `str`
                    The board's kind (public / private / share).
                columns : `list[moncli.entities.Column]`
                    The board's visible columns.
                communication : `str`
                    Get the board communication value - typically meeting ID.
                description : `str`
                    The board's description.
                groups : `list[moncli.entities.Group]`
                    The board's visible groups.
                id : `str`
                    The unique identifier of the board.
                items : `list[moncli.entities.Item]`
                    The board's items (rows).
                name : `str`
                    The board's name.
                owner : `moncli.entities.user.User`
                    The owner of the board.
                permissions : `str`
                    The board's permissions.
                pos : `str`
                    The board's position.
                state : `str`
                    The board's state (all / active / archived / deleted).
                subscribers : `list[moncli.entities.User]`
                    The board's subscribers.
                tags : `list[moncli.entities.Tag]`
                    The board's specific tags.
                top_group : `moncli.entities.Group`
                    The top group at this board.
                updated_at : `str`
                    The last time the board was updated at (ISO8601 DateTime).
                updates : `list[moncli.entities.Update]`
                    The board's updates.
                views : `list[moncli.entities.BoardView]`
                    The board's views.
                workspace : `moncli.entities.Workspace`
                    The workspace that contains this board (null for main workspace).
                workspace_id : `str`
                    The board's workspace unique identifier (null for main workspace).

            Optional Arguments

                workspace_id : `str`
                    Optional workspace id.
                template_id : `str`
                    Optional board template id.
        """
        
        board_data = api.create_board(
            board_name, 
            board_kind, 
            *args,
            api_key=self.__creds.api_key_v2, 
            **kwargs)
        return en.Board(creds=self.__creds, **board_data)


    def get_boards(self, *args, **kwargs):
        """Get a collection of boards.

            Parameters

                args : `tuple`
                    The list of board return fields.
                kwargs : `dict`
                    Optional keyword arguments for getting boards.

            Returns

                boards : `list[moncli.entities.Board]`
                    The collection of boards.

            Return Fields

                activity_logs : `list[moncli.entities.ActivityLog]`
                    The board log events.
                board_folder_id : `int`
                    The board's folder unique identifier.
                board_kind : `str`
                    The board's kind (public / private / share).
                columns : `list[moncli.entities.Column]`
                    The board's visible columns.
                communication : `str`
                    Get the board communication value - typically meeting ID.
                description : `str`
                    The board's description.
                groups : `list[moncli.entities.Group]`
                    The board's visible groups.
                id : `str`
                    The unique identifier of the board.
                items : `list[moncli.entities.Item]`
                    The board's items (rows).
                name : `str`
                    The board's name.
                owner : `moncli.entities.user.User`
                    The owner of the board.
                permissions : `str`
                    The board's permissions.
                pos : `str`
                    The board's position.
                state : `str`
                    The board's state (all / active / archived / deleted).
                subscribers : `list[moncli.entities.User]`
                    The board's subscribers.
                tags : `list[moncli.entities.Tag]`
                    The board's specific tags.
                top_group : `moncli.entities.Group`
                    The top group at this board.
                updated_at : `str`
                    The last time the board was updated at (ISO8601 DateTime).
                updates : `list[moncli.entities.Update]`
                    The board's updates.
                views : `list[moncli.entities.BoardView]`
                    The board's views.
                workspace : `moncli.entities.Workspace`
                    The workspace that contains this board (null for main workspace).
                workspace_id : `str`
                    The board's workspace unique identifier (null for main workspace).

            Optional Arguments

                limit : `int`
                    Number of boards to get; the default is 25.
                page : `int`
                    Page number to get, starting at 1.
                ids : `list[str]`
                    A list of boards unique identifiers.
                board_kind : `moncli.enums.BoardKind`
                    The board's kind (public / private / share)
                state : `moncli.enums.State`
                    The state of the board (all / active / archived / deleted), the default is active.
                order_by : `moncli.enums.BoardsOrderBy`
                    The order in which to retrieve your boards (created_at / used_at). 
        """

        # Add columns to board if column_values are asked for.
        args = list(args)
        for arg in args:
            if 'column_values' in arg:
                args.append('columns.id')
                args.append('columns.type')
                args.append('columns.settings_str')
                break
        
        boards_data = api.get_boards(
            *args, 
            api_key=self.__creds.api_key_v2, 
            **kwargs)

        for data in boards_data:
            try:
                for item_data in data['items']:
                    item_data['board'] = {'id': data['id']}
                    if item_data.__contains__('column_values'):
                        item_data['board']['columns'] = data['columns']
            except:
                break


        return [en.Board(creds=self.__creds, **data) for data in boards_data]


    def get_board(self, id: str = None, name: str = None, *args):
        """Get a board by unique identifier or name.

            Parameters

                id : `str`
                    The unique identifier of the board to retrieve.
                    NOTE: This parameter is mutually exclusive and cannot be used with 'name'.
                name : `str`
                    The name of the board to retrieve.
                    NOTE: This parameter is mutially exclusive and cannot be used with 'id'.
                args : `tuple`
                    The list of board return fields.

            Returns

                board : `moncli.entities.Board`
                    The board requested by ID or name

            Return Fields

                activity_logs : `list[moncli.entities.ActivityLog]`
                    The board log events.
                board_folder_id : `int`
                    The board's folder unique identifier.
                board_kind : `str`
                    The board's kind (public / private / share).
                columns : `list[moncli.entities.Column]`
                    The board's visible columns.
                communication : `str`
                    Get the board communication value - typically meeting ID.
                description : `str`
                    The board's description.
                groups : `list[moncli.entities.Group]`
                    The board's visible groups.
                id : `str`
                    The unique identifier of the board.
                items : `list[moncli.entities.Item]`
                    The board's items (rows).
                name : `str`
                    The board's name.
                owner : `moncli.entities.user.User`
                    The owner of the board.
                permissions : `str`
                    The board's permissions.
                pos : `str`
                    The board's position.
                state : `str`
                    The board's state (all / active / archived / deleted).
                subscribers : `list[moncli.entities.User]`
                    The board's subscribers.
                tags : `list[moncli.entities.Tag]`
                    The board's specific tags.
                top_group : `moncli.entities.Group`
                    The top group at this board.
                updated_at : `str`
                    The last time the board was updated at (ISO8601 DateTime).
                updates : `list[moncli.entities.Update]`
                    The board's updates.
                views : `list[moncli.entities.BoardView]`
                    The board's views.
                workspace : `moncli.entities.Workspace`
                    The workspace that contains this board (null for main workspace).
                workspace_id : `str`
                    The board's workspace unique identifier (null for main workspace).
        """
        
        if id != None and name != None:
            raise MondayClientError('too_many_parameters', "Unable to use both 'id' and 'name' when querying for a board.")
        if id == None and name == None:
            raise MondayClientError('not_enough_parameters', "Either the 'id' or 'name' is required when querying a board.")
        if id != None:         
            return self.get_board_by_id(id)
        else:
            return self.get_board_by_name(name)


    def get_board_by_id(self, id: str, *args):
        """Get a board by unique identifier.

            Parameters

                id : `str`
                    The unique identifier of the board.
                args : `tuple`
                    The list of board return fields.

            Returns

                board : `moncli.entities.Board`
                    The board retrieved by ID.

            Return Fields

                activity_logs : `list[moncli.entities.ActivityLog]`
                    The board log events.
                board_folder_id : `int`
                    The board's folder unique identifier.
                board_kind : `str`
                    The board's kind (public / private / share).
                columns : `list[moncli.entities.Column]`
                    The board's visible columns.
                communication : `str`
                    Get the board communication value - typically meeting ID.
                description : `str`
                    The board's description.
                groups : `list[moncli.entities.Group]`
                    The board's visible groups.
                id : `str`
                    The unique identifier of the board.
                items : `list[moncli.entities.Item]`
                    The board's items (rows).
                name : `str`
                    The board's name.
                owner : `moncli.entities.user.User`
                    The owner of the board.
                permissions : `str`
                    The board's permissions.
                pos : `str`
                    The board's position.
                state : `str`
                    The board's state (all / active / archived / deleted).
                subscribers : `list[moncli.entities.User]`
                    The board's subscribers.
                tags : `list[moncli.entities.Tag]`
                    The board's specific tags.
                top_group : `moncli.entities.Group`
                    The top group at this board.
                updated_at : `str`
                    The last time the board was updated at (ISO8601 DateTime).
                updates : `list[moncli.entities.Update]`
                    The board's updates.
                views : `list[moncli.entities.BoardView]`
                    The board's views.
                workspace : `moncli.entities.Workspace`
                    The workspace that contains this board (null for main workspace).
                workspace_id : `str`
                    The board's workspace unique identifier (null for main workspace).
        """
        
        try:
            board_data = api.get_boards(
                *args,
                api_key=self.__creds.api_key_v2, 
                ids=[int(id)],
                limit=1)[0]
        except IndexError:
            raise MondayClientError('board_not_found', 'Could not find board with ID "{}".'.format(id))
        return en.Board(creds=self.__creds, **board_data)


    def get_board_by_name(self, name: str, *args):
        """Get a board by name.

            Parameters

                name : `str`
                    The name of the board to retrieve.
                args : `tuple`
                    The list of board return fields.

            Returns

                board : `moncli.entities.Board`
                    The board retrieved by name.

            Return Fields

                activity_logs : `list[moncli.entities.ActivityLog]`
                    The board log events.
                board_folder_id : `int`
                    The board's folder unique identifier.
                board_kind : `str`
                    The board's kind (public / private / share).
                columns : `list[moncli.entities.Column]`
                    The board's visible columns.
                communication : `str`
                    Get the board communication value - typically meeting ID.
                description : `str`
                    The board's description.
                groups : `list[moncli.entities.Group]`
                    The board's visible groups.
                id : `str`
                    The unique identifier of the board.
                items : `list[moncli.entities.Item]`
                    The board's items (rows).
                name : `str`
                    The board's name.
                owner : `moncli.entities.user.User`
                    The owner of the board.
                permissions : `str`
                    The board's permissions.
                pos : `str`
                    The board's position.
                state : `str`
                    The board's state (all / active / archived / deleted).
                subscribers : `list[moncli.entities.User]`
                    The board's subscribers.
                tags : `list[moncli.entities.Tag]`
                    The board's specific tags.
                top_group : `moncli.entities.Group`
                    The top group at this board.
                updated_at : `str`
                    The last time the board was updated at (ISO8601 DateTime).
                updates : `list[moncli.entities.Update]`
                    The board's updates.
                views : `list[moncli.entities.BoardView]`
                    The board's views.
                workspace : `moncli.entities.Workspace`
                    The workspace that contains this board (null for main workspace).
                workspace_id : `str`
                    The board's workspace unique identifier (null for main workspace).
        """
        
        # Hard configure the pagination rate.
        page = 1
        page_limit = 500
        record_count = 500

        while record_count >= page_limit:
            boards_data = api.get_boards(
                'id', 'name',
                api_key=self.__creds.api_key_v2, 
                limit=page_limit,
                page=page)
            
            try:
                target_board = [board for board in boards_data if board['name'].lower() == name.lower()][0]
                return self.get_board_by_id(target_board['id'], *args)
            except IndexError:
                page += 1
                record_count = len(boards_data)
                continue
        raise MondayClientError('board_not_found', 'Could not find board with name "{}".'.format(name))   


    def archive_board(self, board_id: str, *args):
        """Archive a board.

            Parameters

                board_id : `str`
                    The board's unique identifier.
                args : `tuple`
                    The list of board return fields.
            
            Returns

                board : `moncli.entities.Board`
                    The archived board.

            Return Fields

                activity_logs : `list[moncli.entities.ActivityLog]`
                    The board log events.
                board_folder_id : `int`
                    The board's folder unique identifier.
                board_kind : `str`
                    The board's kind (public / private / share).
                columns : `list[moncli.entities.Column]`
                    The board's visible columns.
                communication : `str`
                    Get the board communication value - typically meeting ID.
                description : `str`
                    The board's description.
                groups : `list[moncli.entities.Group]`
                    The board's visible groups.
                id : `str`
                    The unique identifier of the board.
                items : `list[moncli.entities.Item]`
                    The board's items (rows).
                name : `str`
                    The board's name.
                owner : `moncli.entities.user.User`
                    The owner of the board.
                permissions : `str`
                    The board's permissions.
                pos : `str`
                    The board's position.
                state : `str`
                    The board's state (all / active / archived / deleted).
                subscribers : `list[moncli.entities.User]`
                    The board's subscribers.
                tags : `list[moncli.entities.Tag]`
                    The board's specific tags.
                top_group : `moncli.entities.Group`
                    The top group at this board.
                updated_at : `str`
                    The last time the board was updated at (ISO8601 DateTime).
                updates : `list[moncli.entities.Update]`
                    The board's updates.
                views : `list[moncli.entities.BoardView]`
                    The board's views.
                workspace : `moncli.entities.Workspace`
                    The workspace that contains this board (null for main workspace).
                workspace_id : `str`
                    The board's workspace unique identifier (null for main workspace).
        """

        board_data = api.archive_board(
            board_id,
            *args, 
            api_key=self.__creds.api_key_v2)
        return en.Board(creds=self.__creds, **board_data)


    def get_assets(self, ids: list, *args):
        """Get a collection of assets by IDs.

            Parameters

                ids : `list[int/str]`
                    Ids of the assets/files you want to get.
                    Example:
                    >>> api.get_assets(ids=['12345678'])
                args : `tuple`
                    The list asset return fields.

            Returns

                assets : `list[moncli.entities.asset.Asset]`
                    A list of file assets uploaded to the account.

            Return Fields

                created_at : `str`
                    The file's creation date.
                file_extension : `str`
                    The file's extension.
                file_size : `int`
                    The file's size in bytes.
                id : `str`
                    The file's unique identifier.
                name : `str`
                    The file's name.
                public_url : `str`
                    Public url to the asset, valid for 1 hour.
                uploaded_by : `moncli.entities.User`
                    The user who uploaded the file
                url : `str`
                    The user who uploaded the file
                url_thumbnail : `str`
                    Url to view the asset in thumbnail mode. Only available for images.
        """
        
        if not ids:
            raise MondayClientError('invalid_parameters', 'File IDs are required.')

        assets_data = api.get_assets(
            ids,
            *args,
            api_key=self.__creds.api_key_v2)
        return [en.asset.Asset(**data) for data in assets_data]


    def get_items(self, get_column_values = None, as_model: type = None, *args, **kwargs):
        """Get a collection of items.

            Parameters

                get_column_values: `bool`
                    Flag used to include column values with the returned items.
                as_model: `type`
                    The MondayModel subclass to be returned.
                args : `tuple`
                    The list of item return fields.
                kwargs : `dict`
                    Optional keyword arguments for querying items.

            Returns

                items : `list[moncli.entities.Item]`
                    The collection of items.

            Return Fields

                assets : `list[moncli.entities.set.Asset]`
                    The item's assets/files.
                board : `moncli.entities.Board`
                    The board that contains this item.
                column_values : `list[moncli.entities.ColumnValue]`
                    The item's column values.
                created_at : `str`
                    The item's create date.
                creator : `moncli.entities.User`
                    The item's creator.
                creator_id : `str`
                    The item's unique identifier.
                group : `moncli.entities.Group`
                    The group that contains this item.
                id : `str`
                    The item's unique identifier.
                name : `str`
                    The item's name.
                state : `str`
                    The board's state (all / active / archived / deleted)
                subscriber : `moncli.entities.User`
                    The pulse's subscribers.
                updated_at : `str`
                    The item's last update date.
                updates : `moncli.entities.Update`
                    The item's updates.

            Optional Arguments

                limit : `int`
                    Number of items to get; the default is 25.
                page : `int`
                    Page number to get, starting at 1.
                ids : `list[str]`
                    A list of items unique identifiers.
                newest_first : `bool`
                    Get the recently created items at the top of the list.
        """

        if get_column_values:
            args = list(args)
            for arg in ['id', 'name', 'column_values.[*]', 'board.id', 'board.columns.[*]']:
                if arg not in args:
                    args.append(arg)

        if kwargs.__contains__('ids'):
            kwargs['ids'] = [int(id) for id in kwargs['ids']]

        items_data = api.get_items(
            *args,
            api_key=self.__creds.api_key_v2, 
            **kwargs)
        items = [en.Item(creds=self.__creds, **item_data) for item_data in items_data] 
        if not as_model:
            return items
        if not issubclass(as_model, MondayModel):
            raise MondayClientError(
                'invalid_as_model_parameter',
                self.id,
                'as_model parameter must be of MondayModel Type'
            )
        return [as_model(item) for item in items]
            

    def get_updates(self, *args, **kwargs):
        """Get a collection of updates.

            Parameters

                args : `tuple`
                    The list of update return fields.
                kwargs : `dict`
                    The optional keyword arguments for querying updates.

            Returns

                updates : `list[moncli.entities.Update]`
                    A collection of updates.

            Return Fields

                assets : `list[moncli.entities.asset.Asset]`
                    The update's assets/files.
                body: `str`
                    The update's html formatted body.
                created_at: `str`
                    The update's creation date.
                creator : `moncli.entities.user.User`
                    The update's creator
                creator_id : `str`
                    The unique identifier of the update creator.
                id : `str`
                    The update's unique identifier.
                item_id : `str`
                    The update's item ID.
                replies : `list[moncli.entities.reply.Reply]
                    The update's replies.
                text_body : `str`
                    The update's text body.
                updated_at : `str`
                    The update's last edit date.

            Optional Arguments

                limit : `int`
                    Number of updates to get; the default is 25.
                page : `int`
                    Page number to get, starting at 1.
        """

        updates_data = api.get_updates(
            *args,
            api_key=self.__creds.api_key_v2, 
            **kwargs)
        return [en.Update(creds=self.__creds, **update_data) for update_data in updates_data]


    def delete_update(self, id: str, *args):
        """Delete an update.

            Parameters

                id : `str`
                    The update's unique identifier.
                args : `tuple`
                    The list of update fields to return.

            Returns

                update : `moncli.entities.Update`
                    The deleted update.

            Return Fields

                assets : `list[moncli.entities.Asset]`
                    The update's assets/files.
                body: `str`
                    The update's html formatted body.
                created_at: `str`
                    The update's creation date.
                creator : `moncli.entities.User`
                    The update's creator
                creator_id : `str`
                    The unique identifier of the update creator.
                id : `str`
                    The update's unique identifier.
                item_id : `str`
                    The update's item ID.
                replies : `list[moncli.entities.Reply]`
                    The update's replies.
                text_body : `str`
                    The update's text body.
                updated_at : `str`
                    The update's last edit date.
        """
        
        update_data = api.delete_update(
            id, 
            *args, 
            api_key=self.__creds.api_key_v2)
        return en.Update(creds=self.__creds, **update_data)


    def clear_item_updates(self, item_id: str, as_model: type = None, *args):
        """Clear an item's updates.

            Parameters

                item_id : `str`
                    The item's unique identifier.
                as_model: `type`
                    The MondayModel subclass to be returned.
                args : `tuple`
                    The list of optional fields to return.

            Returns

                update : `moncli.entities.Item`
                    The updated item.

            Return Fields

                assets : `list[moncli.entities.asset.Asset]`
                    The item's assets/files.
                board : `moncli.entities.board.Board`
                    The board that contains this item.
                column_values : `list[moncli.entities.column_value.ColumnValue]`
                    The item's column values.
                created_at : `str`
                    The item's create date.
                creator : `moncli.entities.user.User`
                    The item's creator.
                creator_id : `str`
                    The item's unique identifier.
                group : `moncli.entities.group.Group`
                    The group that contains this item.
                id : `str`
                    The item's unique identifier.
                name : `str`
                    The item's name.
                state : `str`
                    The board's state (all / active / archived / deleted)
                subscriber : `moncli.entities.user.User`
                    The pulse's subscribers.
                updated_at : `str`
                    The item's last update date.
                updates : `moncli.entities.update.Update`
                    The item's updates.
        """
        
        item_data = api.clear_item_updates(
            item_id,
            *args,
            api_key=self.__creds.api_key_v2)
        items= en.Item(creds=self.__creds, **item_data)
        if as_model:
            if not issubclass(as_model, MondayModel):
                raise MondayClientError(
                    'invalid_as_model_parameter',
                    self.id,
                    'as_model parameter must be of MondayModel Type'
                )
            return [as_model(item) for item in items]
        else:
            return items


    def create_notification(self, text: str, user_id: str, target_id: str, target_type: NotificationTargetType, *args, **kwargs):
        """Create a new notification.

            Parameters

                text : `str`
                    The notification text.
                user_id : `str`
                    The user's unique identifier.
                target_id : `str`
                    The target's unique identifier.
                target_type : `moncli.enums.NotificationTargetType`
                    The target's type (Project / Post)
                args : `tuple`
                    The list of notification return fields.
                kwargs : `dict`
                    The optional keyword arguments for creating notifications.

            Returns

                notification : `moncli.entities.Notification`
                    A new notification.

            Return Fields

                id : `str`
                    The notification's unique identifier.
                text : `str`
                    The notification text.

            Optional Arguments

                payload : `json`
                    The notification payload.
        """

        notification_data = api.create_notification(
            text, 
            user_id, 
            target_id,
            target_type,
            *args,
            api_key=self.__creds.api_key_v2, 
            **kwargs)
        return en.Notification(notification_data)
    

    def create_or_get_tag(self, tag_name: str, *args, **kwargs):
        """Create a new tag or get it if it already exists.

            Parameters

                tag_name : `str`
                    The new tag's name.
                args : `tuple`
                    The list of tag return fields.
                kwargs : `dict`
                    Optional keyword arguments for creating/getting a tag.

            Returns

                tag : `moncli.entities.Tag`
                    The created or retrieved tag.

            Return Fields

                color : `str`
                    The tag's color.
                id : `str`
                    The tag's unique identifier.
                name : `str`
                    The tag's name.

            Optional Arguments

                board_id : `str`
                    The private board id to create the tag at (not needed for public boards).
        """

        tag_data = api.create_or_get_tag(
            tag_name,
            *args,
            api_key=self.__creds.api_key_v2, 
            **kwargs)
        return en.Tag(tag_data)


    def get_tags(self, *args, **kwargs):
        """Get a collection of tags.

            Parameters

                args : `tuple`
                    The list of tag return fields.
                kwargs : `dict`
                    Optional keyword arguments for querying tags.

            Returns

                tags : `list[moncli.entities.Tag]`
                    A collection of tags.

            Return Fields

                color : `str`
                    The tag's color.
                id : `str`
                    The tag's unique identifier.
                name : `str`
                    The tag's name.

            Optional Arguments

                ids : `list[str]`
                    The list of tags unique identifiers.
        """
        
        tags_data = api.get_tags(
            *args,
            api_key=self.__creds.api_key_v2,
            **kwargs)
        return [en.Tag(tag_data) for tag_data in tags_data]
    

    def get_users(self, *args, **kwargs):
        """Get a collection of users.

            Parameters

                args : `tuple`
                    The list of user return fields.
                kwargs : `dict`
                    Optional keyword arguments for querying users.

            Returns

                users : `list[moncli.entities.User]`
                    A collection of users.

            Return Fields

                account : `moncli.entities.Account`
                    The user's account.
                birthday : `str`
                    The user's birthday.
                country_code : `str`
                    The user's country code.
                created_at : `str`
                    The user's creation date.
                email : `str`
                    The user's email.
                enabled : `bool`
                    Is the user enabled or not.
                id : `str`
                    The user's unique identifier.
                is_admin: `bool`
                    Is the user a admin or not.
                is_guest : `bool`
                    Is the user a guest or not.
                is_pending : `bool`
                    Is the user a pending user.
                is_verified: `bool`
                    Is the user is verified
                is_view_only : `bool`
                    Is the user a view only user or not.
                join_date : `str`
                    The date the user joined the account.
                location : `str`
                    The user' location.
                mobile_phone : `str`
                    The user's mobile phone number.
                name : `str`
                    The user's name.
                phone : `str`
                    The user's phone number.
                photo_original : `str`
                    The user's photo in the original size.
                photo_small : `str`
                    The user's photo in small size (150x150).
                photo_thumb : `str`
                    The user's photo in thumbnail size (100x100).
                photo_thumb_small : `str`
                    The user's photo in small thumbnail size (50x50).
                photo_tiny : `str`
                    The user's photo in tiny size (30x30).
                teams : `list[moncli.entities.Team]`
                    The teams the user is a member in.
                time_zone_identifier : `str`
                    The user's time zone identifier.
                title : `str`
                    The user's title.
                url : `str`
                    The user's profile url.
                utc_hours_diff : `int`
                    The user's UTC hours difference.

            Optional Arguments

                ids : `list[str]`
                    A list of users unique identifiers.
                kind : `moncli.entities.UserKind`
                    The kind to search users by (all / non_guests / guests / non_pending).
                newest_first : `bool`
                    Get the recently created users at the top of the list.
                limit : `int`
                    Number of users to get.
        """

        users_data = api.get_users(
            *args,
            api_key=self.__creds.api_key_v2, 
            **kwargs)
        return [en.User(creds=self.__creds, **user_data) for user_data in users_data]
    

    def get_teams(self, *args, **kwargs):
        """Get a collection of teams.

            Parameters

                args : `tuple`
                    The list of team return fields.
                kwargs : `dict`
                    Optional keyword arguments for querying teams.
            
            Returns

                teams : `list[moncli.entities.Team]`
                    A collection of teams.

            Return Fields

                id : `int`
                    The team's unique identifier.
                name : `str`
                    The team's name.
                picture_url : `str`
                    The team's picture url.
                users : `moncli.entities.user.User`
                    The users in the team.

            
            Optional Arguments

                ids : `list[str]`
                    A list of teams unique identifiers.
        """

        teams_data = api.get_teams(
            *args,
            api_key=self.__creds.api_key_v2,
            **kwargs)
        return [en.Team(creds=self.__creds, **team_data) for team_data in teams_data]
    

    def get_me(self, *args):
        """Get the connected user's information.

            Parameters

                args : `tuple`
                    The list of user return fields.
            
            Returns

                user : `moncli.entities.User`
                    The connected user.

            Return Fields

                account : `moncli.entities.Account`
                    The user's account.
                birthday : `str`
                    The user's birthday.
                country_code : `str`
                    The user's country code.
                created_at : `str`
                    The user's creation date.
                email : `str`
                    The user's email.
                enabled : `bool`
                    Is the user enabled or not.
                id : `str`
                    The user's unique identifier.
                is_admin: `bool`
                    Is the user a admin or not.
                is_guest : `bool`
                    Is the user a guest or not.
                is_pending : `bool`
                    Is the user a pending user.
                is_verified: `bool`
                    Is the user is verified
                is_view_only : `bool`
                    Is the user a view only user or not.
                join_date : `str`
                    The date the user joined the account.
                location : `str`
                    The user' location.
                mobile_phone : `str`
                    The user's mobile phone number.
                name : `str`
                    The user's name.
                phone : `str`
                    The user's phone number.
                photo_original : `str`
                    The user's photo in the original size.
                photo_small : `str`
                    The user's photo in small size (150x150).
                photo_thumb : `str`
                    The user's photo in thumbnail size (100x100).
                photo_thumb_small : `str`
                    The user's photo in small thumbnail size (50x50).
                photo_tiny : `str`
                    The user's photo in tiny size (30x30).
                teams : `list[moncli.entities.Team]`
                    The teams the user is a member in.
                time_zone_identifier : `str`
                    The user's time zone identifier.
                title : `str`
                    The user's title.
                url : `str`
                    The user's profile url.
                utc_hours_diff : `int`
                    The user's UTC hours difference.
        """

        user_data = api.get_me(
            *args, 
            api_key=self.__creds.api_key_v2)

        return en.User(creds=self.__creds, **user_data)


    def get_account(self, *args):
        """Get the connected user's account.

            Parameters

                args : `tuple`
                    The list of account return fields.
            
            Returns

                account : `moncli.entities.Account`
                    The connected user's account.

            Return Fields

                first_day_of_the_week : `str`
                    The first day of the week for the account (sunday / monday).
                id : `int`
                    The account's unique identifier.
                logo : `str`
                    The account's logo.
                name : `str`
                    The account's name.
                plan : `moncli.entities.Plan`
                    The account's payment plan.
                show_timeline_weekends : `bool`
                    Show weekends in timeline.
                slug : `str`
                    The account's slug.
        """

        account_data = api.get_account(
            *args,
            api_key=self.__creds.api_key_v2)

        return en.Account(creds=self.__creds, **account_data)


    def create_workspace(self, name: str, kind: WorkspaceKind, *args, **kwargs):
        """Creates a new workspace

            Parameters

                name : `str`
                    The workspace's name.
                kind : `moncli.enums.WorkspaceKind`
                    The workspace's kind (open / closed)
                args : `tuple`
                    The list of workspace return fields.
                kwargs : `dict`
                    Optional keyword arguments listed below.
            
            Returns

                workspace : `moncli.entities.Workspace`
                    The newly created workspace

            Return Fields

                id : `str`
                    The workspace's unique identifier
                name : `str`
                    The workspace's name
                kind : `str`
                    The workspace's kind (open / closed)
                description : `str`
                    The workspace's description

            Optional Arguments

                description : `str`
                    The workspace's description.
        """
        
        workspace_data = api.create_workspace(
            name, 
            kind,
            *args,
            api_key=self.__creds.api_key_v2,
            **kwargs)

        return en.Workspace(workspace_data)
    
    def add_users_to_workspace(self, workspace_id: str, user_ids: list, kind: WorkspaceSubscriberKind, *args, **kwargs):
        """Allows you to add users to a workspace. You can define if users will be added as regular subscribers or as owners of the workspace.

            Parameters 

                workspace_id: int
                     The workspace's unique identifier
                user_ids: `list[str]`
                        User IDs to subscribe to the workspace
                kind: WorkspaceSUbscriberKind
                    Kind of subscribers added (subscriber/owner)
                args: tuple
                    The collection of workspace return fields.
                kwargs : `dict`
                    Optional keyword arguments listed below.
            
            Return Fields
            
                account : `moncli.entities.Account`
                    The user's account.
                birthday : `str`
                    The user's birthday.
                country_code : `str`
                    The user's country code.
                created_at : `str`
                    The user's creation date.
                email : `str`
                    The user's email.
                enabled : `bool`
                    Is the user enabled or not.
                id : `str`
                    The user's unique identifier.
                is_admin: `bool`
                    Is the user a admin or not.
                is_guest : `bool`
                    Is the user a guest or not.
                is_pending : `bool`
                    Is the user a pending user.
                is_verified: `bool`
                    Is the user is verified
                is_view_only : `bool`
                    Is the user a view only user or not.
                join_date : `str`
                    The date the user joined the account.
                location : `str`
                    The user' location.
                mobile_phone : `str`
                    The user's mobile phone number.
                name : `str`
                    The user's name.
                phone : `str`
                    The user's phone number.
                photo_original : `str`
                    The user's photo in the original size.
                photo_small : `str`
                    The user's photo in small size (150x150).
                photo_thumb : `str`
                    The user's photo in thumbnail size (100x100).
                photo_thumb_small : `str`
                    The user's photo in small thumbnail size (50x50).
                photo_tiny : `str`
                    The user's photo in tiny size (30x30).
                teams : `list[moncli.entities.Team]`
                    The teams the user is a member in.
                time_zone_identifier : `str`
                    The user's time zone identifier.
                title : `str`
                    The user's title.
                url : `str`
                    The user's profile url.
                utc_hours_diff : `int`
                    The user's UTC hours difference.
         """
         
        users_data = api.add_users_to_workspace(
            workspace_id,
            user_ids, 
            kind,
            *args,
            api_key=self.__creds.api_key_v2,
            **kwargs)

        return [en.User(creds=self.__creds, **data) for data in users_data]


    def delete_users_from_workspace(self, workspace_id: str, user_ids: list(), *args: tuple, **kwargs: dict ):
        """Allows you to delete users to a workspace.

            Parameters 

                workspace_id: int
                    The workspace's unique identifier
                user_ids: `list[str]`
                        User IDs to subscribe to the workspace
                *args: tuple
                    The collection of workspace return fields.
            
            Return Fields
        
                account : `moncli.entities.Account`
                    The user's account.
                birthday : `str`
                    The user's birthday.
                country_code : `str`
                    The user's country code.
                created_at : `str`
                    The user's creation date.
                email : `str`
                    The user's email.
                enabled : `bool`
                    Is the user enabled or not.
                id : `str`
                    The user's unique identifier.
                is_admin: `bool`
                    Is the user a admin or not.
                is_guest : `bool`
                    Is the user a guest or not.
                is_pending : `bool`
                    Is the user a pending user.
                is_verified: `bool`
                    Is the user is verified
                is_view_only : `bool`
                    Is the user a view only user or not.
                join_date : `str`
                    The date the user joined the account.
                location : `str`
                    The user' location.
                mobile_phone : `str`
                    The user's mobile phone number.
                name : `str`
                    The user's name.
                phone : `str`
                    The user's phone number.
                photo_original : `str`
                    The user's photo in the original size.
                photo_small : `str`
                    The user's photo in small size (150x150).
                photo_thumb : `str`
                    The user's photo in thumbnail size (100x100).
                photo_thumb_small : `str`
                    The user's photo in small thumbnail size (50x50).
                photo_tiny : `str`
                    The user's photo in tiny size (30x30).
                teams : `list[moncli.entities.Team]`
                    The teams the user is a member in.
                time_zone_identifier : `str`
                    The user's time zone identifier.
                title : `str`
                    The user's title.
                url : `str`
                    The user's profile url.
                utc_hours_diff : `int`
                    The user's UTC hours difference.
        """
        
        users_data = api.delete_users_from_workspace(
            workspace_id,
            user_ids,
            *args,
            api_key=self.__creds.api_key_v2,
            **kwargs)

        return [en.User(creds=self.__creds, **data) for data in users_data]


    def add_teams_to_workspace(self, workspace_id: str, team_ids: list(), *args, **kwargs ):
        """Allows you to add teams to a workspace.

            Parameters

                workspace_id: str
                    The workspace's unique identifier.
                team_ids: list()
                    Team ids to subscribe to the workspace.
            
            Returns

                teams: `list[moncli.entities.user.Team]`
                    List of teams deleted from the workspace
            
            Return Fields

                id : `int`
                    The team's unique identifier.
                name : `str`
                    The team's name.
                picture_url : `str`
                    The team's picture url.
                users : `moncli.entities.user.User`
                    The users in the team.
                name : `str`
                    The team's name.
                picture_url : `str`
                    The team's picture url.
                users : `moncli.entities.user.User`
                    The users in the team.

            Optional Arguments

                api_key : `str`
                    The monday.com v2 API user key.
        """
        
        teams_data = api.add_teams_to_workspace(
            workspace_id,
            team_ids,
            *args,
            api_key=self.__creds.api_key_v2,
            **kwargs)

        return [en.Team(creds=self.__creds, **data) for data in teams_data]


    def delete_teams_from_workspace(self, workspace_id: str, team_ids: list(), *args, **kwargs ):
        """Allows you to delete teams from a workspace.

            Parameters

                workspace_id: str
                    The workspace's unique identifier.
                team_ids: list()
                    Team ids to subscribe to the workspace.
            
            Returns

                teams: `list[moncli.entities.user.Team]`
                    List of teams deleted from the workspace
            
            Return Fields

                id : `int`
                    The team's unique identifier.
                name : `str`
                    The team's name.
                picture_url : `str`
                    The team's picture url.
                users : `moncli.entities.user.User`
                    The users in the team.
                name : `str`
                    The team's name.
                picture_url : `str`
                    The team's picture url.
                users : `moncli.entities.user.User`
                    The users in the team.

            Optional Arguments

                api_key : `str`
                    The monday.com v2 API user key.
        """
        
        teams_data = api.delete_teams_from_workspace(
            workspace_id,
            team_ids,
            *args,
            api_key=self.__creds.api_key_v2,
            **kwargs)

        return [en.Team(creds=self.__creds, **data) for data in teams_data]


class AuthorizationError(Exception):
    def __init__(self, user_name: str):
        self.message = 'User {} was not recognized by the applied token'.format(user_name)


class TooManyGetBoardParameters(Exception):
    def __init__(self):
        self.message = "Unable to use both 'id' and 'name' when querying for a board."
        

class NotEnoughGetBoardParameters(Exception):
    def __init__(self):
        self.message = "Either the 'id' or 'name' is required when querying a board."


class BoardNotFound(Exception):
    def __init__(self, search_type, value):        
        if search_type == 'id':
            self.message = 'Unable to find board with name: "{}".'.format(value)       
        elif search_type == 'name':
            self.message = 'Unable to find board with the ID: "{}".'.format(value)
        else:
            self.message = 'Unable to find the requested board.'

class AssetIdsRequired(Exception):
    def __init__(self):
        self.message = "Ids parameter is required."




from schematics.models import Model
from schematics import types

from .. import api, entities as en
from ..enums import *


class _User(Model):
    """User base model"""

    id = types.StringType(required=True)
    name = types.StringType()
    created_at = types.StringType()
    url = types.StringType()
    email = types.StringType()
    enabled = types.BooleanType()
    birthday = types.StringType()
    country_code = types.StringType()
    is_admin = types.BooleanType()
    is_guest = types.BooleanType()
    is_pending = types.BooleanType()
    is_verified = types.BooleanType() 
    is_view_only = types.BooleanType()
    join_date = types.StringType()
    location = types.StringType()
    mobile_phone = types.StringType()
    phone = types.StringType()
    photo_original = types.StringType()
    photo_thumb = types.StringType()
    photo_tiny = types.StringType()
    time_zone_identifier = types.StringType()
    title = types.StringType()
    utc_hours_diff = types.IntType()


class User(_User):
    """A monday.com user
    
        Properties
        
            account : `moncli.entities.user.Account`
                The user's account.
            birthday : `str`
                The user's birthday.
            country_code : `str`
                The user's country code.
            created_at : `str`
                The user's creation date.
            email : `str`
                The user's email.
            enabled : `bool`
                Is the user enabled or not.
            id : `str`
                The user's unique identifier.
            is_admin: `bool`
                Is the user a admin or not.
            is_guest : `bool`
                Is the user a guest or not.
            is_pending : `bool`
                Is the user a pending user.
            is_verified: `bool`
                Is the user is verified
            is_view_only : `bool`
                Is the user a view only user or not.
            join_date : `str`
                The date the user joined the account.
            location : `str`
                The user' location.
            mobile_phone : `str`
                The user's mobile phone number.
            name : `str`
                The user's name.
            phone : `str`
                The user's phone number.
            photo_original : `str`
                The user's photo in the original size.
            photo_small : `str`
                The user's photo in small size (150x150).
            photo_thumb : `str`
                The user's photo in thumbnail size (100x100).
            photo_thumb_small : `str`
                The user's photo in small thumbnail size (50x50).
            photo_tiny : `str`
                The user's photo in tiny size (30x30).
            teams : `list[moncli.entities.user.Team]`
                The teams the user is a member in.
            time_zone_identifier : `str`
                The user's time zone identifier.
            title : `str`
                The user's title.
            url : `str`
                The user's profile url.
            utc_hours_diff : `int`
                The user's UTC hours difference.

        Methods

            get_account : `moncli.entities.Account`
                Get the user's account.
            get_teams : `list[moncli.entities.Team]`
                Get teams the user is a member in.
            send_notification : `moncli.entities.objects.Notification`
                Create a new notification.
    """

    def __init__(self, **kwargs):
        self.__creds = kwargs.pop('creds')
        self.__account = None
        account = kwargs.pop('account', None)
        if account:
            self.__account = Account(creds=self.__creds, **account)
        self.__teams = None
        teams = kwargs.pop('teams', None)
        if teams != None:
            self.__teams = [Team(creds=self.__creds, **team) for team in teams]
        super(User, self).__init__(kwargs)

    @property
    def account(self):
        """The user's account"""

        if not self.__account:
            self.__account = self.get_account()
        return self.__account

    @property
    def teams(self):
        """The teams the user is a member in."""

        if not self.__teams:
            self.__teams = self.get_teams()
        return self.__teams

    def __repr__(self):
        o = self.to_primitive()       
        if self.__account:
            o['account'] = self.__account
        if self.__teams:
            o['teams'] = [team.to_primitive() for team in self.__teams]
        return str(o)


    def get_account(self, *args):
        """Get the user's account.

            Parameters

                args : `tuple`
                    The list of account return fields.
            
            Returns

                account : `moncli.entities.Account`
                    The user's account.

            Return Fields

                first_day_of_the_week : `str`
                    The first day of the week for the account (sunday / monday).
                id : `int`
                    The account's unique identifier.
                logo : `str`
                    The account's logo.
                name : `str`
                    The account's name.
                plan : `moncli.entities.Plan`
                    The account's payment plan.
                show_timeline_weekends : `bool`
                    Show weekends in timeline.
                slug : `str`
                    The account's slug.
        """

        account_data = api.get_users(
            *api.get_field_list(api.DEFAULT_ACCOUNT_QUERY_FIELDS, 'account', *args),
            api_key=self.__creds.api_key_v2, 
            ids=[int(self.id)])[0]['account']
        return Account(
            creds=self.__creds,
            **account_data)
    

    def get_teams(self, *args):
        """Get teams the user is a member in.

            Parameters

                args : `tuple`
                    The list of team return fields.

        
            Returns

                teams : `list[moncli.entities.Team]`
                    Teams the user is a member in.

            Return Fields

                id : `int`
                    The team's unique identifier.
                name : `str`
                    The team's name.
                picture_url : `str`
                    The team's picture url.
                users : `moncli.entities.user.User`
                    The users in the team.
        """

        teams_data = api.get_users(
            *api.get_field_list(api.DEFAULT_TEAM_QUERY_FIELDS, 'teams', *args),
            api_key=self.__creds.api_key_v2,
            ids=[int(self.id)])[0]['teams']
        return [Team(creds=self.__creds, **team_data) for team_data in teams_data]
    

    def send_notification(self, text: str, target_id: str, target_type: NotificationTargetType, *args, **kwargs):
        """Create a new notification.

            Parameters

                text : `str`
                    The notification text.
                user_id : `str`
                    The user's unique identifier.
                target_id : `str`
                    The target's unique identifier.
                target_type : `moncli.enums.NotificationTargetType`
                    The target's type (Project / Post)
                args : `tuple`
                    The list of noficiation return fields.
                kwargs : `dict`
                    Optional keyword arguments.

            Returns

                notification : `moncli.entities.ActivityLog`
                    The new notification.

            Return Fields

                id : `str`
                    The notification's unique identifier.
                text : `str`
                    The notification text.
            
            Optional Arguments

                payload : `json`
                    The notification payload.
        """

        notification_data = api.create_notification(
            text, 
            self.id, 
            target_id, 
            target_type, 
            *args, 
            api_key=self.__creds.api_key_v2, 
            **kwargs)
        return en.Notification(notification_data)


class _Team(Model):
    """Team base model."""

    id = types.StringType(required=True)
    name = types.StringType()
    picture_url = types.StringType()


class Team(_Team):
    """A team of users.
    
        Properties

            id : `int`
                The team's unique identifier.
            name : `str`
                The team's name.
            picture_url : `str`
                The team's picture url.
            users : `moncli.entities.User`
                The users in the team.

        Methods

            get_users : `list[moncli.entities.User]`
                Get the users in the team.
    """

    def __init__(self, **kwargs):
        self.__creds = kwargs.pop('creds')
        self.__users = None
        users = kwargs.pop('users', None)
        if users:
            self.__users = [User(creds=self.__creds, **user) for user in users]
        super(Team, self).__init__(kwargs)

    @property
    def users(self):
        """The users in the team."""

        if not self.__users:
            self.__users = self.get_users()
        return self.__users

    def __repr__(self):
        o = self.to_primitive()
        if self.__users:
            o['users'] = self.__users
        return str(o)

    def get_users(self, *args, **kwargs):
        """Get the users in the team.
        
            Parameters

                args : `tuple`
                    The list of user return fields.
                kwargs : `dict`
                    Optional keyword arguments for querying the team's users.

            Returns

                users : `list[moncli.entities.User]`
                    The users in the team.

            Return Fields

                account : `moncli.entities.Account`
                    The user's account.
                birthday : `str`
                    The user's birthday.
                country_code : `str`
                    The user's country code.
                created_at : `str`
                    The user's creation date.
                email : `str`
                    The user's email.
                enabled : `bool`
                    Is the user enabled or not.
                id : `str`
                    The user's unique identifier.
                is_admin: `bool`
                    Is the user a admin or not.
                is_guest : `bool`
                    Is the user a guest or not.
                is_pending : `bool`
                    Is the user a pending user.
                is_verified: `bool`
                    Is the user is verified
                is_view_only : `bool`
                    Is the user a view only user or not.
                join_date : `str`
                    The date the user joined the account.
                location : `str`
                    The user' location.
                mobile_phone : `str`
                    The user's mobile phone number.
                name : `str`
                    The user's name.
                phone : `str`
                    The user's phone number.
                photo_original : `str`
                    The user's photo in the original size.
                photo_small : `str`
                    The user's photo in small size (150x150).
                photo_thumb : `str`
                    The user's photo in thumbnail size (100x100).
                photo_thumb_small : `str`
                    The user's photo in small thumbnail size (50x50).
                photo_tiny : `str`
                    The user's photo in tiny size (30x30).
                teams : `list[moncli.entities.Team]`
                    The teams the user is a member in.
                time_zone_identifier : `str`
                    The user's time zone identifier.
                title : `str`
                    The user's title.
                url : `str`
                    The user's profile url.
                utc_hours_diff : `int`
                    The user's UTC hours difference.
            
            Optional Arguments

                ids : `list[int]`
                    A list of users unique identifiers.
                kind : `moncli.enums.UserKind`
                    The kind to search users by (all / non_guests / guests / non_pending)
                newest_first : `bool`
                    Get the recently created users at the top of the list.
                limit : `int`
                    Number of users to get.            
        """

        if kwargs:
            kwargs = {'users': kwargs}
        users_data = api.get_teams(
            *api.get_field_list(api.DEFAULT_USER_QUERY_FIELDS, 'users', *args),
            api_key=self.__creds.api_key_v2, 
            ids=[int(self.id)],
            **kwargs)[0]['users']
        return [User(creds=self.__creds, **user_data) for user_data in users_data]


class _Account(Model):
    """Account base model."""

    id = types.StringType(required=True)
    name = types.StringType()
    first_day_of_the_week = types.StringType()
    logo = types.StringType()
    show_timeline_weekends = types.BooleanType()
    slug = types.StringType()


class Account(_Account):
    """Your monday.com account.

        Properties

            first_day_of_the_week : `str`
                The first day of the week for the account (sunday / monday).
            id : `int`
                The account's unique identifier.
            logo : `str`
                The account's logo.
            name : `str`
                The account's name.
            plan : `moncli.entities.Plan`
                The account's payment plan.
            show_timeline_weekends : `bool`
                Show weekends in timeline.
            slug : `str`
                The account's slug.

        Methods

            get_plan : `moncli.entities.Plan`
                Get the account's payment plan.
    """

    def __init__(self, **kwargs):
        self.__creds = kwargs.pop('creds')
        self.__plan = None
        plan = kwargs.pop('plan', None)
        if plan:
            self.__plan = en.Plan(plan)
        super(Account, self).__init__(kwargs)

    @property
    def plan(self):
        """A payment plan."""
        if not self.__plan:
            self.__plan = self.get_plan()
        return self.__plan

    @property
    def first_day_of_the_week_enum(self):
        """The first day of the week for the account (sunday / monday)."""

        if self.first_day_of_the_week:
            return FirstDayOfTheWeek[self.first_day_of_the_week]

    def __repr__(self):
        o = self.to_primitive()
        if self.__plan:
            o['plan'] = self.__plan
        return str(o)

    def get_plan(self, *args):
        """Get the account's payment plan.
        
            Parameters

                args : `tuple`
                    The list of plan optional return fields.
            
            Returns

                plan : `moncli.entities.Plan`
                    The account's payment plan.
            
            Return Fields

                max_users : `int`
                    The maximum users allowed in the plan.
                period : `str`
                    The plan's time period.
                tier : `str`
                    The plan's tier.
                version : `int`
                    The plan's version.
        """
        
        plan_data = api.get_account(
            *api.get_field_list(api.DEFAULT_PLAN_QUERY_FIELDS, 'plan', *args),
            api_key=self.__creds.api_key_v2)['plan']
        return en.Plan(plan_data)


try:
    from setuptools import setup
except:
    from distutils.core import setup

config = {
    'description': 'Moncli, a pythonic/DDD client for Monday.com',
    'author': 'Andrew Shatz, Trix Solutions',
    'url': r'https://github.com/trix-solutions/moncli',
    'download_url': r'https://github.com/trix-solutions/moncli',
    'author_email': 'andrew.shatz@trix.solutions',
    'version': '2.0.0',
    'install_requires': [
        'nose>=1.3.7',
        'requests>=2.24.0',
        'pytz>=2020.1',
        'pycountry>=20.7.3',
        'deprecated>=1.2.10',
        'schematics>=2.1.0'
    ],
    'tests_require': [
        'nose>=1.3.7'
    ],
    'packages': [
        'moncli',
        'moncli.api_v1',
        'moncli.api_v2',
        'moncli.entities'
    ],    
    'scripts': [],
    'name': 'moncli'
}

setup(**config)



import importlib, json, pickle

from schematics.exceptions import DataError
from schematics.models import Model

class MondayModel(Model):

    def __init__(self, item = None, raw_data: dict = None, id: str = None, name: str = None, board = None, *args, **kwargs):
        
        self._item = item
        self._board = board
        self._module = self.__module__
        self._class = self.__class__.__name__
        self._original_values = {}

        has_original = False

        if item:
            self.id = item.id
            self.name = item.name
            raw_data = {}
            column_values = item.column_values
            # When loading column values to the model fields,
            for field, field_type in self._fields.items():
                # First give title a try, as it is best known.
                try:
                    raw_data[field] = column_values[field_type.metadata['title']]
                except:
                    # If that doesn't work, then see if there is an ID.
                    try:
                        raw_data[field] = column_values[field_type.metadata['id']]
                    except:
                        # Otherwise, it's not work the trouble...
                        continue
            has_original = True
        elif raw_data:
            self.id = raw_data.pop('id', None)
            self.name = raw_data.pop('name', None)
        elif (name and id):
            self.id = id
            self.name = name        
        else:
            raise TypeError('Input item, raw data, or id and name are required.')
                
        super(MondayModel, self).__init__(raw_data=raw_data)
        
        if board:
            columns = board.columns
            for field, field_type in self._fields.items():
                try:
                    column = columns[field_type.metadata['title']]
                except:
                    try:
                        column = columns[field_type.metadata['id']]
                    except:
                        continue
                field_type.metadata['id'] = column.id
                field_type.metadata['title'] = column.title
                settings = json.loads(column.settings_str)
                for k, v in settings.items():
                    field_type.metadata[k] = v

        for field, _type in self._fields.items():
            if has_original:
                self._original_values[field] = pickle.dumps(getattr(self, field))
            else:
                self._original_values[field] = pickle.dumps(_type.default)


    @property
    def item(self):
        return self._item

    def __repr__(self):
        return str(self.__dict__)

    def to_primitive(self, diff_only = False, role = None, app_data = None, **kwargs):        
        base_dict = super().to_primitive(role=role, app_data=app_data, **kwargs)
        result = {}

        def value_changed(value, other):
            return value != other

        for field, value in base_dict.items():
            model_field = self._fields[field]
            if model_field.is_readonly:
                continue      
            if not value:
                value = model_field.null_value
            new_value = pickle.dumps(getattr(self, field))
            if diff_only and not value_changed(new_value, self._original_values[field]):
                continue
            try:
                result[model_field.metadata['id']] = value
            except:
                result[field] = value

        return result

    def get_metadata(self, field: str, key: str):
        try:
            field_obj = self._fields[field]
        except KeyError:
            raise KeyError('Model does not contain field: ({}).'.format(field))
        try:
            return field_obj.metadata[key]
        except KeyError:
            raise KeyError('Model field does not contain metadata key: ({}).'.format(key))
        

    def save(self, group = None, archive: bool = False):
        if not self._item and not self._board:
            raise TypeError('Unable to save model without monday.com item/board information.')

        if self._board and not self.name:
            raise TypeError('Unable to save new model as item without a name.')
        
        try:
            self.validate()
        except DataError as ex:
            raise ex

        column_values = self.to_primitive(diff_only=True)
        if self._item:
            if column_values:
                self._item = self._item.change_multiple_column_values(column_values, get_column_values=True)
            if group:
                self._item = self._item.move_to_group(group.id, get_column_values=True)
        elif self._board and column_values:
            if group:
                self._item = self._board.add_item(self.name, get_column_values=True, group_id=group.id, column_values=column_values)
            else:
                self._item = self._board.add_item(self.name, get_column_values=True, column_values=column_values)
        if archive:
            self._item.archive()
        return getattr(importlib.import_module(self._module), self._class)(self._item)


from typing import List, Dict, Any

from ..enums import BoardKind, ColumnType, NotificationTargetType, WebhookEventType, WorkspaceKind, WorkspaceSubscriberKind
from . import graphql as gql
from .constants import *
from .requests import execute_query, upload_file


def create_board(board_name: str, board_kind: BoardKind, *args, **kwargs):
    """Create a new board.

        Parameters

            board_name : `str`
                The board's name.
            board_kind : `moncli.enums.BoardKind`
                The board's kind (public / private / share)
            args : `tuple`
                The list of board return fields.
            kwargs : `dict`
                Optional keyword arguments for creating boards.

        Returns

            board_data : `dict`
                A monday.com board in dictionary form.

        Return Fields
        
            activity_logs : `list[moncli.entities.ActivityLog]`
                The board log events.
            board_folder_id : `int`
                The board's folder unique identifier.
            board_kind : `str`
                The board's kind (public / private / share).
            columns : `list[moncli.entities.Column]`
                The board's visible columns.
            communication : `str`
                Get the board communication value - typically meeting ID.
            description : `str`
                The board's description.
            groups : `list[moncli.entities.Group]`
                The board's visible groups.
            id : `str`
                The unique identifier of the board.
            items : `list[moncli.entities.Item]`
                The board's items (rows).
            name : `str`
                The board's name.
            owner : `moncli.entities.user.User`
                The owner of the board.
            permissions : `str`
                The board's permissions.
            pos : `str`
                The board's position.
            state : `str`
                The board's state (all / active / archived / deleted).
            subscribers : `list[moncli.entities.User]`
                The board's subscribers.
            tags : `list[moncli.entities.Tag]`
                The board's specific tags.
            top_group : `moncli.entities.Group`
                The top group at this board.
            updated_at : `str`
                The last time the board was updated at (ISO8601 DateTime).
            updates : `list[moncli.entities.Update]`
                The board's updates.
            views : `list[moncli.entities.BoardView]`
                The board's views.
            workspace : `moncli.entities.Workspace`
                The workspace that contains this board (null for main workspace).
            workspace_id : `str`
                The board's workspace unique identifier (null for main workspace).

        Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
            workspace_id : `int`
                Optional workspace id.
            template_id : `int`
                Optional board template id.
    """

    kwargs['board_name'] = gql.StringValue(board_name)
    kwargs['board_kind'] = gql.EnumValue(board_kind)
    return execute_query(kwargs.pop('api_key', None), query_name=CREATE_BOARD, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def get_boards(*args, **kwargs) -> List[Dict[str, Any]]:
    """Retrieves a list of boards.

        Parameters
        
            args : `tuple`
                The list of board return fields.
            kwargs : `dict`
                Optional keyword arguments for querying boards.

        Returns

            data : `dict`
                A monday.com board in dictionary form.

        Return Fields
        
            activity_logs : `list[moncli.entities.ActivityLog]`
                The board log events.
            board_folder_id : `int`
                The board's folder unique identifier.
            board_kind : `str`
                The board's kind (public / private / share).
            columns : `list[moncli.entities.Column]`
                The board's visible columns.
            communication : `str`
                Get the board communication value - typically meeting ID.
            description : `str`
                The board's description.
            groups : `list[moncli.entities.Group]`
                The board's visible groups.
            id : `str`
                The unique identifier of the board.
            items : `list[moncli.entities.Item]`
                The board's items (rows).
            name : `str`
                The board's name.
            owner : `moncli.entities.user.User`
                The owner of the board.
            permissions : `str`
                The board's permissions.
            pos : `str`
                The board's position.
            state : `str`
                The board's state (all / active / archived / deleted).
            subscribers : `list[moncli.entities.User]`
                The board's subscribers.
            tags : `list[moncli.entities.Tag]`
                The board's specific tags.
            top_group : `moncli.entities.Group`
                The top group at this board.
            updated_at : `str`
                The last time the board was updated at (ISO8601 DateTime).
            updates : `list[moncli.entities.Update]`
                The board's updates.
            views : `list[moncli.entities.BoardView]`
                The board's views.
            workspace : `moncli.entities.Workspace`
                The workspace that contains this board (null for main workspace).
            workspace_id : `str`
                The board's workspace unique identifier (null for main workspace).

        Optional Arguments
            
            api_key : `str`
                The monday.com v2 API user key.
            limit : `int`
                Number of boards to get; the default is 25.
            page : `int`
                Page number to get, starting at 1.
            ids : `list[str]`
                A list of boards unique identifiers.
            board_kind : `moncli.enumns.BoardKind`
                The board's kind (public / private /share).
            state : `moncli.enums.State`
                The state of the board (all / active / archived / deleted),; the default is active.
            order_by : `moncli.enums.BoardsOrderBy`
                The order in which to retrieve your boards (created_at / used_at).       
    """
    
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=BOARDS, operation_type=gql.OperationType.QUERY, fields=args, arguments=kwargs)  


def archive_board(board_id: str, *args, **kwargs):
    """Archives a board.

        Parameters
        

            board_id : `str`
                The board's unique identifier.
            args : `tuple`
                The list of board return fields.
            kwargs : `dict`
                Optional keyword arguments for archiving boards.

        Returns
            
            data : `dict`
                A monday.com board in dictionary form.

        Return Fields
        
            activity_logs : `list[moncli.entities.ActivityLog]`
                The board log events.
            board_folder_id : `int`
                The board's folder unique identifier.
            board_kind : `str`
                The board's kind (public / private / share).
            columns : `list[moncli.entities.Column]`
                The board's visible columns.
            communication : `str`
                Get the board communication value - typically meeting ID.
            description : `str`
                The board's description.
            groups : `list[moncli.entities.Group]`
                The board's visible groups.
            id : `str`
                The unique identifier of the board.
            items : `list[moncli.entities.Item]`
                The board's items (rows).
            name : `str`
                The board's name.
            owner : `moncli.entities.user.User`
                The owner of the board.
            permissions : `str`
                The board's permissions.
            pos : `str`
                The board's position.
            state : `str`
                The board's state (all / active / archived / deleted).
            subscribers : `list[moncli.entities.User]`
                The board's subscribers.
            tags : `list[moncli.entities.Tag]`
                The board's specific tags.
            top_group : `moncli.entities.Group`
                The top group at this board.
            updated_at : `str`
                The last time the board was updated at (ISO8601 DateTime).
            updates : `list[moncli.entities.Update]`
                The board's updates.
            views : `list[moncli.entities.BoardView]`
                The board's views.
            workspace : `moncli.entities.Workspace`
                The workspace that contains this board (null for main workspace).
            workspace_id : `str`
                The board's workspace unique identifier (null for main workspace).

        Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
    """

    kwargs['board_id'] = gql.IntValue(board_id)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=ARCHIVE_BOARD, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def add_subscribers_to_board(board_id: str, user_ids: list, *args, **kwargs):
    """Add subscribers to a board.

        Parameters

            board_id : `str`
                The board's unique identifier.
            user_ids : `list[str]`
                User ids to subscribe to a board.
            args : `tuple`
                The list of user return fields.
            kwargs : `dict`
                Optional keyword arguments for adding board subscribers.

        Returns

            data : `dict`
                A monday.com user in dictionary form.
        
        Return Fields
        
            account : `moncli.entities.Account`
                The user's account.
            birthday : `str`
                The user's birthday.
            country_code : `str`
                The user's country code.
            created_at : `str`
                The user's creation date.
            email : `str`
                The user's email.
            enabled : `bool`
                Is the user enabled or not.
            id : `str`
                The user's unique identifier.
            is_guest : `bool`
                Is the user a guest or not.
            is_pending : `bool`
                Is the user a pending user.
            is_view_only : `bool`
                Is the user a view only user or not.
            join_date : `str`
                The date the user joined the account.
            location : `str`
                The user' location.
            mobile_phone : `str`
                The user's mobile phone number.
            name : `str`
                The user's name.
            phone : `str`
                The user's phone number.
            photo_original : `str`
                The user's photo in the original size.
            photo_small : `str`
                The user's photo in small size (150x150).
            photo_thumb : `str`
                The user's photo in thumbnail size (100x100).
            photo_thumb_small : `str`
                The user's photo in small thumbnail size (50x50).
            photo_tiny : `str`
                The user's photo in tiny size (30x30).
            teams : `list[moncli.entities.Team]`
                The teams the user is a member in.
            time_zone_identifier : `str`
                The user's time zone identifier.
            title : `str`
                The user's title.
            url : `str`
                The user's profile url.
            utc_hours_diff : `int`
                The user's UTC hours difference.

        Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
            kind : `moncli.enums.SubscriberKind`
                Subscribers kind (subscriber / owner).
    """

    kwargs['board_id'] = gql.IntValue(board_id)
    kwargs['user_ids'] = gql.ListValue([int(id) for id in user_ids])
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=ADD_SUBSCRIBERS_TO_BOARD, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def delete_subscribers_from_board(board_id: str, user_ids: list, *args, **kwargs):
    """Remove subscribers from the board.

        Parameters

            board_id : `str`
                The board's unique identifier.
            user_ids : `list[str]`
                User ids to unsubscribe from board.
            args : `tuple`
                The list of user return fields.
            kwargs : `dict`
                Additional keyword arguments for deleting subscribers.

        Returns

            data : `dict`
                A monday.com user in dictionary form.
        
        Return Fields
        
            account : `moncli.entities.Account`
                The user's account.
            birthday : `str`
                The user's birthday.
            country_code : `str`
                The user's country code.
            created_at : `str`
                The user's creation date.
            email : `str`
                The user's email.
            enabled : `bool`
                Is the user enabled or not.
            id : `str`
                The user's unique identifier.
            is_guest : `bool`
                Is the user a guest or not.
            is_pending : `bool`
                Is the user a pending user.
            is_view_only : `bool`
                Is the user a view only user or not.
            join_date : `str`
                The date the user joined the account.
            location : `str`
                The user' location.
            mobile_phone : `str`
                The user's mobile phone number.
            name : `str`
                The user's name.
            phone : `str`
                The user's phone number.
            photo_original : `str`
                The user's photo in the original size.
            photo_small : `str`
                The user's photo in small size (150x150).
            photo_thumb : `str`
                The user's photo in thumbnail size (100x100).
            photo_thumb_small : `str`
                The user's photo in small thumbnail size (50x50).
            photo_tiny : `str`
                The user's photo in tiny size (30x30).
            teams : `list[moncli.entities.Team]`
                The teams the user is a member in.
            time_zone_identifier : `str`
                The user's time zone identifier.
            title : `str`
                The user's title.
            url : `str`
                The user's profile url.
            utc_hours_diff : `int`
                The user's UTC hours difference.

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
    """

    kwargs = {
        'board_id': gql.IntValue(board_id),
        'user_ids': gql.ListValue([int(id) for id in user_ids])
    }
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=DELETE_SUBSCRIBERS_FROM_BOARD, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)

    
def create_column(board_id: str, title: str, column_type: ColumnType, *args, **kwargs):
    """Create a new column in board.

        Parameters
        
            board_id : `str`
                The board's unique identifier.
            title : `str`
                The new column's title.
            args : `tuple`
                The list of column return fields.
            kwargs : `dict`
                Optional keyword arguments for creating columns.

        Returns
            
            data : `dict`
                A monday.com column in dictionary form.

        Return Fields
            
            archived : `bool`
                Is the column archived or not.
            id : `str`
                The column's unique identifier.
            pos : `str`
                The column's position in the board.
            settings_str : `str`
                The column's settings in a string form.
            title : `str`
                The column's title.
            type : `str`
                The column's type.
            width : `int`
                The column's width.

        Optional Arguments
            
            api_key : `str`
                The monday.com v2 API user key.
            column_type : `moncli.enumns.ColumnType`
                The type of column to create.
            defaults : `json`
                The new column's defaults.
    """
    
    kwargs['board_id'] = gql.IntValue(board_id)
    kwargs['title'] = gql.StringValue(title)
    kwargs['column_type'] = gql.EnumValue(column_type)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=CREATE_COLUMN, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def change_column_title(title: str, column_id: str, board_id: str, *args, **kwargs):
    """Change an item's column title.

        Parameters
        
            title : `str`
                The new title of the column.
            column_id : `str`
                The column's unique identifier.
            board_id : `str`
                The board's unique identifier.
            args : `tuple`
                The list of item return fields.
            kwargs : `dict`
                Optional arguments for changing a column value.

        Returns
            
            data : `dict`
                A monday.com column in item form.

        Return Fields
        
            archived : `bool`
                Is the column archived or not.
            id : `str`
                The column's unique identifier.
            pos : `str`
                The column's position in the board. 
            settings_str : `str`
                The column's settings in a string form.
            settings : `moncli.entities.Settings`
                The settings in entity form (status / dropdown)
            title : `str`
                The column's title.
            type : `str`
                The column's type.
            width : `int`
                The column's width.
            
        Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
    """
    
    kwargs['column_id'] = gql.StringValue(column_id)
    kwargs['board_id'] = gql.IntValue(board_id)
    kwargs['title'] = gql.StringValue(title)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=CHANGE_COLUMN_TITLE, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def change_column_value(item_id: str, column_id: str, board_id: str, value: str, *args, **kwargs):
    """Change an item's column value.

        Parameters
        
            item_id : `str`
                The item's unique identifier.
            column_id : `str`
                The column's unique identifier.
            board_id : `str`
                The board's unique identifier.
            value : `json`
                The new value of the column.
            args : `tuple`
                The list of item return fields.
            kwargs : `dict`
                Optional arguments for changing a column value.

        Returns
            
            data : `dict`
                A monday.com column in item form.

        Return Fields
        
            assets : `list[moncli.entities.Asset]`
                The item's assets/files.
            board : `moncli.entities.Board`
                The board that contains this item.
            column_values : `list[moncli.entities.ColumnValue]`
                The item's column values.
            created_at : `str`
                The item's create date.
            creator : `moncli.entities.User`
                The item's creator.
            creator_id : `str`
                The item's unique identifier.
            group : `moncli.entities.Group`
                The group that contains this item.
            id : `str`
                The item's unique identifier.
            name : `str`
                The item's name.
            state : `str`
                The board's state (all / active / archived / deleted)
            subscriber : `moncli.entities.User`
                The pulse's subscribers.
            updated_at : `str`
                The item's last update date.
            updates : `moncli.entities.Update`
                The item's updates.
        
         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
    """
    
    kwargs['item_id'] = gql.IntValue(item_id)
    kwargs['column_id'] = gql.StringValue(column_id)
    kwargs['board_id'] = gql.IntValue(board_id)
    kwargs['value'] = gql.JsonValue(value)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=CHANGE_COLUMN_VALUE, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)

def change_simple_column_value(item_id: str, board_id: str, column_id: str, value: str, *args, **kwargs):
    """Changes the column valuesvalues using simple values

        Parameters
        
            item_id : str
                The item's identifier.
            column_id : str
                The column identifier on your board.
            board_id : str
                The board identifier.
            value : str
                The new simple value of the column.
            *args : tuple
                The collection of item return fields.

        Returns
            
            data : `dict`
                A monday.com column in item form.

        Return Fields
        
            assets : `list[moncli.entities.Asset]`
                The item's assets/files.
            board : `moncli.entities.Board`
                The board that contains this item.
            column_values : `list[moncli.entities.ColumnValue]`
                The item's column values.
            created_at : `str`
                The item's create date.
            creator : `moncli.entities.User`
                The item's creator.
            creator_id : `str`
                The item's unique identifier.
            group : `moncli.entities.Group`
                The group that contains this item.
            id : `str`
                The item's unique identifier.
            name : `str`
                The item's name.
            state : `str`
                The board's state (all / active / archived / deleted)
            subscriber : `moncli.entities.User`
                The pulse's subscribers.
            updated_at : `str`
                The item's last update date.
            updates : `moncli.entities.Update`
                The item's updates.
        
         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
    """
    
    kwargs['item_id'] = gql.IntValue(item_id)
    kwargs['board_id'] = gql.IntValue(board_id)
    kwargs['column_id'] = gql.StringValue(column_id)
    kwargs['value']=gql.StringValue(value)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=CHANGE_SIMPLE_COLUMN_VALUE, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def change_multiple_column_value(item_id: str, board_id: str, column_values: dict, *args, **kwargs):
    """Changes the column values of a specific item.

        Parameters
        
            item_id : `str`
                The item's unique identifier.
            board_id : `str`
                The board's unique identifier.
            column_values : `json`
                The column values updates.
            args : `tuple`
                The list of item return fields.
            kwargs : `dict`
                Optional arguments for changing a column value.

        Returns
            
            data : `dict`
                A monday.com column in item form.

        Return Fields
        
            assets : `list[moncli.entities.Asset]`
                The item's assets/files.
            board : `moncli.entities.Board`
                The board that contains this item.
            column_values : `list[moncli.entities.ColumnValue]`
                The item's column values.
            created_at : `str`
                The item's create date.
            creator : `moncli.entities.User`
                The item's creator.
            creator_id : `str`
                The item's unique identifier.
            group : `moncli.entities.Group`
                The group that contains this item.
            id : `str`
                The item's unique identifier.
            name : `str`
                The item's name.
            state : `str`
                The board's state (all / active / archived / deleted)
            subscriber : `moncli.entities.User`
                The pulse's subscribers.
            updated_at : `str`
                The item's last update date.
            updates : `moncli.entities.Update`
                The item's updates.
        
         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
    """
    
    kwargs['item_id'] = gql.IntValue(item_id)
    kwargs['board_id'] = gql.IntValue(board_id)
    kwargs['column_values'] = gql.JsonValue(column_values)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=CHANGE_MULTIPLE_COLUMN_VALUES, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def get_assets(ids: list, *args, **kwargs):
    """Get a collection of assets by ids.

        Parameters

            ids : `list[str]`
                Ids of the assets/files you want to get.
            args : `tuple`
                The list of asset return fields.
            kwargs : `dict`
                Optional arguments for querying assets.

        Returns
            
            data : `dict`
                A monday.com asset in item form.

        Return Fields
        
            created_at : `str`
                The file's creation date.
            file_extension : `str`
                The file's extension.
            file_size : `int`
                The file's size in bytes.
            id : `str`
                The file's unique identifier.
            name : `str`
                The file's name.
            public_url : `str`
                Public url to the asset, valid for 1 hour.
            uploaded_by : `moncli.entities.user.User`
                The user who uploaded the file
            url : `str`
                The user who uploaded the file
            url_thumbnail : `str`
                Url to view the asset in thumbnail mode. Only available for images.  

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key. 
    """
    
    ids = [gql.IntValue(id).value for id in ids]
    kwargs['ids'] = gql.ListValue(ids)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=ASSETS, operation_type=gql.OperationType.QUERY, *args, fields=args, arguments=kwargs)


def duplicate_group(board_id: str, group_id: str, *args, **kwargs):
    """Duplicate a group.

        Parameters
        
            board_ id : `str`
                The board's unique identifier.
            group_id : `str`
                The group's unique identifier.
            args : `tuple`
                The list of asset return fields.
            kwargs : `dict`
                Optional arguments for querying assets.

        Returns
            
            data : `dict`
                A monday.com group in item form.

        Return Fields
        
            archived : `bool`
                Is the group archived or not.
            color : `str`
                The group's color.
            deleted : `bool`
                Is the group deleted or not.
            id : `str`
                The group's unique identifier.
            items : `list[moncli.entities.Item]`
                The items in the group.
            position : `str`
                The group's position in the board.
            title : `str`
                The group's title.

        
        Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
            add_to_top : `bool`
                Should the new group be added to the top.
            group_title : `str`
                The group's title.
    """
    
    kwargs['board_id'] = gql.IntValue(board_id)
    kwargs['group_id'] = gql.StringValue(group_id)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=DUPLICATE_GROUP, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def create_group(board_id: str, group_name: str, *args, **kwargs):
    """Creates a new group in a specific board.

        Parameters
        
            board_id : `str`
                The board's unique identifier.
            group_name : `str`
                The name of the new group.
            args : `tuple`
                The list of group return fields.
            kwargs : `dict`
                Optional arguments for querying assets.

        Returns
            
            data : `dict`
                A monday.com group in item form.

        Return Fields
        
            archived : `bool`
                Is the group archived or not.
            color : `str`
                The group's color.
            deleted : `bool`
                Is the group deleted or not.
            id : `str`
                The group's unique identifier.
            items : `list[moncli.entities.Item]`
                The items in the group.
            position : `str`
                The group's position in the board.
            title : `str`
                The group's title.


         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
    """
    
    kwargs['board_id'] = gql.IntValue(board_id)
    kwargs['group_name'] = gql.StringValue(group_name)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=CREATE_GROUP, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def archive_group(board_id: str, group_id: str, *args, **kwargs):
    """Archives a group in a specific board.

        Parameters

            board_id : `str`
                The board's unique identifier.
            group_id : `str`
                The group's unique identifier.
            args : `tuple`
                The list of group return fields.
            kwargs : `dict`
                Optional arguments for archiving groups.

        Returns
            
            data : `dict`
                A monday.com group in item form.

        Return Fields
        
            archived : `bool`
                Is the group archived or not.
            color : `str`
                The group's color.
            deleted : `bool`
                Is the group deleted or not.
            id : `str`
                The group's unique identifier.
            items : `list[moncli.entities.Item]`
                The items in the group.
            position : `str`
                The group's position in the board.
            title : `str`
                The group's title.
        
         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
    """
    
    kwargs['board_id'] = gql.IntValue(board_id)
    kwargs['group_id'] = gql.StringValue(group_id)    
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=ARCHIVE_GROUP, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def delete_group(board_id: str, group_id: str, *args, **kwargs):
    """Deletes a group in a specific board.

        Parameters

            board_id : `str`
                The board's unique identifier.
            group_id : `str`
                The group's unique identifier.
            args : `tuple`
                The list of group return fields.
            kwargs : `dict`
                Optional arguments for deleting groups.

        Returns
            
            data : `dict`
                A monday.com group in item form.

        Return Fields
        
            archived : `bool`
                Is the group archived or not.
            color : `str`
                The group's color.
            deleted : `bool`
                Is the group deleted or not.
            id : `str`
                The group's unique identifier.
            items : `list[moncli.entities.Item]`
                The items in the group.
            position : `str`
                The group's position in the board.
            title : `str`
                The group's title.
        
         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
    """
    
    kwargs['board_id'] = gql.IntValue(board_id)
    kwargs['group_id'] = gql.StringValue(group_id)       
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=DELETE_GROUP, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def create_item(item_name: str, board_id: str, *args, **kwargs):
    """Create a new item.

        Parameters
        
            item_name : `str`
                The new item's name.
            board_id : `str`
                The board's unique identifier.
            args : `tuple`
                The list of item return fields.
            kwargs : `dict`
                Optional arguments for creating items.

        Returns
                    
            data : `dict`
                A monday.com column in item form.

        Return Fields
        
            assets : `list[moncli.entities.Asset]`
                The item's assets/files.
            board : `moncli.entities.Board`
                The board that contains this item.
            column_values : `list[moncli.entities.ColumnValue]`
                The item's column values.
            created_at : `str`
                The item's create date.
            creator : `moncli.entities.User`
                The item's creator.
            creator_id : `str`
                The item's unique identifier.
            group : `moncli.entities.Group`
                The group that contains this item.
            id : `str`
                The item's unique identifier.
            name : `str`
                The item's name.
            state : `str`
                The board's state (all / active / archived / deleted)
            subscriber : `moncli.entities.User`
                The pulse's subscribers.
            updated_at : `str`
                The item's last update date.
            updates : `moncli.entities.Update`
                The item's updates.

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.        
            group_id : `str`
                The group's unique identifier.
            column_values : `json`
                The column values of the new item.
    """
    
    kwargs['item_name'] = gql.StringValue(item_name)
    kwargs['board_id'] = gql.IntValue(board_id)    
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=CREATE_ITEM, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def create_subitem(parent_item_id: str, item_name: str, *args, **kwargs):
    """Create subitem.

        Parameters
        
            parent_item_id : `str`
                The parent item's unique identifier.
            item_name : `str`
                The new item's name.
            args : `tuple`
                The list of item return fields.
            kwargs : `dict`
                Optional arguments for creating subitems.

        Returns
                    
            data : `dict`
                A monday.com column in item form.

        Return Fields
        
            assets : `list[moncli.entities.Asset]`
                The item's assets/files.
            board : `moncli.entities.Board`
                The board that contains this item.
            column_values : `list[moncli.entities.ColumnValue]`
                The item's column values.
            created_at : `str`
                The item's create date.
            creator : `moncli.entities.User`
                The item's creator.
            creator_id : `str`
                The item's unique identifier.
            group : `moncli.entities.Group`
                The group that contains this item.
            id : `str`
                The item's unique identifier.
            name : `str`
                The item's name.
            state : `str`
                The board's state (all / active / archived / deleted)
            subscriber : `moncli.entities.User`
                The pulse's subscribers.
            updated_at : `str`
                The item's last update date.
            updates : `moncli.entities.Update`
                The item's updates.

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
            column_values : `json`
                The column values of the new item.
    """

    kwargs['parent_item_id'] = gql.IntValue(parent_item_id)
    kwargs['item_name'] = gql.StringValue(item_name)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=CREATE_SUBITEM, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def get_items(*args, **kwargs):
    """Get a collection of items.

        Parameters
        
            args : `tuple`
                The list of item return fields.
            kwargs : `dict`
                Optional arguments for querying items.

        Returns
            
            data : `dict`
                A monday.com column in item form.

        Return Fields
        
            assets : `list[moncli.entities.Asset]`
                The item's assets/files.
            board : `moncli.entities.Board`
                The board that contains this item.
            column_values : `list[moncli.entities.ColumnValue]`
                The item's column values.
            created_at : `str`
                The item's create date.
            creator : `moncli.entities.User`
                The item's creator.
            creator_id : `str`
                The item's unique identifier.
            group : `moncli.entities.Group`
                The group that contains this item.
            id : `str`
                The item's unique identifier.
            name : `str`
                The item's name.
            state : `str`
                The board's state (all / active / archived / deleted)
            subscriber : `moncli.entities.User`
                The pulse's subscribers.
            updated_at : `str`
                The item's last update date.
            updates : `moncli.entities.Update`
                The item's updates.

        Optional Arguments
        
            api_key : `str`
                The monday.com v2 API user key.
            limit : `int`
                Number of items to get; the default is 25.
            page : `int`
                Page number to get, starting at 1.
            ids : `list[str]`
                A list of items unique identifiers.
            mewest_first : `bool`
                Get the recently created items at the top of the list.
    """
    
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=ITEMS, operation_type=gql.OperationType.QUERY, fields=args, arguments=kwargs)


def get_items_by_column_values(board_id: str, column_id: str, column_value: str, *args, **kwargs):
    """Search items by a value for a single column.

        Parameters
        
            board_id : `str`
                The board's unique identifier.
            column_id : `str`
                The column's unique identifier.
            column_value `str`
                The column value to search items by.
            args : `tuple`
                The list of item return fields.
            kwargs : `dict`
                Optional arguments for querying items by column value.

        Returns
            
            data : `dict`
                A monday.com column in item form.

        Return Fields
        
            assets : `list[moncli.entities.Asset]`
                The item's assets/files.
            board : `moncli.entities.Board`
                The board that contains this item.
            column_values : `list[moncli.entities.ColumnValue]`
                The item's column values.
            created_at : `str`
                The item's create date.
            creator : `moncli.entities.User`
                The item's creator.
            creator_id : `str`
                The item's unique identifier.
            group : `moncli.entities.Group`
                The group that contains this item.
            id : `str`
                The item's unique identifier.
            name : `str`
                The item's name.
            state : `str`
                The board's state (all / active / archived / deleted)
            subscriber : `moncli.entities.User`
                The pulse's subscribers.
            updated_at : `str`
                The item's last update date.
            updates : `moncli.entities.Update`
                The item's updates.

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
            limit : `int`
                Number of items to get.
            page : `int`
                Page number to get, starting at 1.
            column_type : `str`
                The column type.
            state : `moncli.enums.State`
                The state of the item (all / active / archived / deleted); the default is active.
    """
    
    kwargs['board_id'] = gql.IntValue(board_id)
    kwargs['column_id'] = gql.StringValue(column_id)
    kwargs['column_value'] = gql.StringValue(column_value)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=ITEMS_BY_COLUMN_VALUES, operation_type=gql.OperationType.QUERY, fields=args, arguments=kwargs)


def get_items_by_multiple_column_values(board_id: str, column_id: str, column_value: list, *args, **kwargs):
    """Search items by multiple values for a single column.

        Parameters
        
            board_id : `str`
                The board's unique identifier.
            column_id : `str`
                The column's unique identifier.
            column_values `list`
                The column value to search items by.
            args : `tuple`
                The list of item return fields.
            kwargs : `dict`
                Optional arguments for querying items by column value.

        Returns
            
            data : `dict`
                A monday.com column in item form.

        Return Fields
        
            assets : `list[moncli.entities.Asset]`
                The item's assets/files.
            board : `moncli.entities.Board`
                The board that contains this item.
            column_values : `list[moncli.entities.ColumnValue]`
                The item's column values.
            created_at : `str`
                The item's create date.
            creator : `moncli.entities.User`
                The item's creator.
            creator_id : `str`
                The item's unique identifier.
            group : `moncli.entities.Group`
                The group that contains this item.
            id : `str`
                The item's unique identifier.
            name : `str`
                The item's name.
            state : `str`
                The board's state (all / active / archived / deleted)
            subscriber : `moncli.entities.User`
                The pulse's subscribers.
            updated_at : `str`
                The item's last update date.
            updates : `moncli.entities.Update`
                The item's updates.

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
            limit : `int`
                Number of items to get.
            page : `int`
                Page number to get, starting at 1.
            column_type : `str`
                The column type.
            state : `moncli.enums.State`
                The state of the item (all / active / archived / deleted); the default is active.
    """
    
    kwargs['board_id'] = gql.IntValue(board_id)
    kwargs['column_id'] = gql.StringValue(column_id)
    kwargs['column_values'] = gql.ListValue(column_value)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=ITEMS_BY_MULTIPLE_COLUMN_VALUES, operation_type=gql.OperationType.QUERY, fields=args, arguments=kwargs)


def clear_item_updates(item_id: str, *args, **kwargs):
    """Clear an item's updates.

        Parameters
        
            item_id : `str`
                The item's unique identifier.
            args : `tuple`
                The list of item return fields.
            kwargs : `dict`
                Optional arguments for clearing item updates.

        Returns
            
            data : `dict`
                A monday.com column in item form.

        Return Fields
        
            assets : `list[moncli.entities.Asset]`
                The item's assets/files.
            board : `moncli.entities.Board`
                The board that contains this item.
            column_values : `list[moncli.entities.ColumnValue]`
                The item's column values.
            created_at : `str`
                The item's create date.
            creator : `moncli.entities.User`
                The item's creator.
            creator_id : `str`
                The item's unique identifier.
            group : `moncli.entities.Group`
                The group that contains this item.
            id : `str`
                The item's unique identifier.
            name : `str`
                The item's name.
            state : `str`
                The board's state (all / active / archived / deleted)
            subscriber : `moncli.entities.User`
                The pulse's subscribers.
            updated_at : `str`
                The item's last update date.
            updates : `moncli.entities.Update`
                The item's updates.

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
    """
    
    kwargs['item_id'] = gql.IntValue(item_id)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=CLEAR_ITEM_UPDATES, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def move_item_to_group(item_id: str, group_id: str, *args, **kwargs):
    """Move an item to a different group.

        Parameters
        
            item_id : `str`
                The item's unique identifier.
            group_id : `str`
                The group's unique identifier.
            args : `tuple`
                The list of item return fields.
            kwargs : `dict`
                Optional arguments for moving item to group.

        Returns
            
            data : `dict`
                A monday.com column in item form.

        Return Fields
        
            assets : `list[moncli.entities.Asset]`
                The item's assets/files.
            board : `moncli.entities.Board`
                The board that contains this item.
            column_values : `list[moncli.entities.ColumnValue]`
                The item's column values.
            created_at : `str`
                The item's create date.
            creator : `moncli.entities.User`
                The item's creator.
            creator_id : `str`
                The item's unique identifier.
            group : `moncli.entities.Group`
                The group that contains this item.
            id : `str`
                The item's unique identifier.
            name : `str`
                The item's name.
            state : `str`
                The board's state (all / active / archived / deleted)
            subscriber : `moncli.entities.User`
                The pulse's subscribers.
            updated_at : `str`
                The item's last update date.
            updates : `moncli.entities.Update`
                The item's updates.
        
         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
    """
    
    kwargs['item_id'] = gql.IntValue(item_id)
    kwargs['group_id'] = gql.StringValue(group_id)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=MOVE_ITEM_TO_GROUP, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def archive_item(item_id: str, *args, **kwargs):
    """Archive an item.

        Parameters
        
            item_id : `str`
                The item's unique identifier.
            args : `tuple`
                The list of item return fields.
            kwargs : `dict`
                Optional arguments for archiving items.

        Returns
            
            data : `dict`
                A monday.com column in item form.

        Return Fields
        
            assets : `list[moncli.entities.Asset]`
                The item's assets/files.
            board : `moncli.entities.Board`
                The board that contains this item.
            column_values : `list[moncli.entities.ColumnValue]`
                The item's column values.
            created_at : `str`
                The item's create date.
            creator : `moncli.entities.User`
                The item's creator.
            creator_id : `str`
                The item's unique identifier.
            group : `moncli.entities.Group`
                The group that contains this item.
            id : `str`
                The item's unique identifier.
            name : `str`
                The item's name.
            state : `str`
                The board's state (all / active / archived / deleted)
            subscriber : `moncli.entities.User`
                The pulse's subscribers.
            updated_at : `str`
                The item's last update date.
            updates : `moncli.entities.Update`
                The item's updates.

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
    """
    
    kwargs['item_id'] = gql.IntValue(item_id)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=ARCHIVE_ITEM, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def delete_item(item_id: str, *args, **kwargs):
    """Delete an item.

        Parameters
        
            item_id : `str`
                The item's unique identifier.
            args : `tuple`
                The list of item return fields.
            kwargs : `dict`
                Optional arguments for item.

        Returns
            
            data : `dict`
                A monday.com item in dictionary form.

        Return Fields
        
            assets : `list[moncli.entities.Asset]`
                The item's assets/files.
            board : `moncli.entities.Board`
                The board that contains this item.
            column_values : `list[moncli.entities.ColumnValue]`
                The item's column values.
            created_at : `str`
                The item's create date.
            creator : `moncli.entities.User`
                The item's creator.
            creator_id : `str`
                The item's unique identifier.
            group : `moncli.entities.Group`
                The group that contains this item.
            id : `str`
                The item's unique identifier.
            name : `str`
                The item's name.
            state : `str`
                The board's state (all / active / archived / deleted)
            subscriber : `moncli.entities.User`
                The pulse's subscribers.
            updated_at : `str`
                The item's last update date.
            updates : `moncli.entities.Update`
                The item's updates.

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
    """
    
    kwargs['item_id'] = gql.IntValue(item_id)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=DELETE_ITEM, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def duplicate_item(board_id: str, item_id: str, *args, **kwargs):
    """Duplicate an item.

        Parameters

            board_id : `str`
                The board's unique identifier.
            item_id : `str`
                The item's unique identifier.
            args : `tuple`
                The list of item return fields.
            kwargs : `dict`
                Optional arguments for item.

        Returns
            
            data : `dict`
                A monday.com item in dictionary form.

        Return Fields
        
            assets : `list[moncli.entities.Asset]`
                The item's assets/files.
            board : `moncli.entities.Board`
                The board that contains this item.
            column_values : `list[moncli.entities.ColumnValue]`
                The item's column values.
            created_at : `str`
                The item's create date.
            creator : `moncli.entities.User`
                The item's creator.
            creator_id : `str`
                The item's unique identifier.
            group : `moncli.entities.Group`
                The group that contains this item.
            id : `str`
                The item's unique identifier.
            name : `str`
                The item's name.
            state : `str`
                The board's state (all / active / archived / deleted)
            subscriber : `moncli.entities.User`
                The pulse's subscribers.
            updated_at : `str`
                The item's last update date.
            updates : `moncli.entities.Update`
                The item's updates.

        Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
            with_updates : `bool`
                Duplicate with the item's updates.
    """
    
    kwargs = {
        'board_id': gql.IntValue(board_id),
        'item_id': gql.IntValue(item_id)
    }
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=DUPLICATE_ITEM, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def create_update(body: str, item_id: str, *args, **kwargs):
    """Create a new update.

        Parameters
        
            body : `str`
                The update text.
            item_id : `str`
                The item's unique identifier.
            args : `tuple`
                The list of update return fields.
            kwargs : `dict`
                Optional arguments for creating updates.

        Returns
            
            data : `dict`
                A monday.com column in dictionary form.

        Return Fields
        
            assets : `list[moncli.entities.Asset]`
                The update's assets/files.
            body: `str`
                The update's html formatted body.
            created_at: `str`
                The update's creation date.
            creator : `moncli.entities.User`
                The update's creator
            creator_id : `str`
                The unique identifier of the update creator.
            id : `str`
                The update's unique identifier.
            item_id : `str`
                The update's item ID.
            replies : `list[moncli.entities.Reply]
                The update's replies.
            text_body : `str`
                The update's text body.
            updated_at : `str`
                The update's last edit date.

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.        
            parent_id : `str`
                The parent post identifier.
    """
    
    kwargs['body'] = gql.StringValue(body)
    kwargs['item_id'] = gql.IntValue(item_id)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=CREATE_UPDATE, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def get_updates(*args, **kwargs):
    """Get a collection of updates.

        Parameters
        
            args : `tuple`
                The list of update return fields.
            kwargs : `dict`
                Optional arguments for querying updates.

        Returns
            
            data : `dict`
                A monday.com column in dictionary form.

        Return Fields
        
            assets : `list[moncli.entities.Asset]`
                The update's assets/files.
            body: `str`
                The update's html formatted body.
            created_at: `str`
                The update's creation date.
            creator : `moncli.entities.User`
                The update's creator
            creator_id : `str`
                The unique identifier of the update creator.
            id : `str`
                The update's unique identifier.
            item_id : `str`
                The update's item ID.
            replies : `list[moncli.entities.Reply]
                The update's replies.
            text_body : `str`
                The update's text body.
            updated_at : `str`
                The update's last edit date.

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.        
            limit : `int`
                Number of updates to get; the default is 25.
            page : `int`
                Page number to get, starting at 1.
    """
    
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=UPDATES, operation_type=gql.OperationType.QUERY, fields=args, arguments=kwargs)


def delete_update(id: str, *args, **kwargs):
    """Delete an update.

        Parameters
        
            id : `int`
                The update's unique identifier.
            args : `tuple`
                The list of update return fields.
            kwargs : `dict`
                Optional arguments for deleting updates.

        Returns
            
            data : `dict`
                A monday.com column in dictionary form.

        Return Fields
        
            assets : `list[moncli.entities.Asset]`
                The update's assets/files.
            body: `str`
                The update's html formatted body.
            created_at: `str`
                The update's creation date.
            creator : `moncli.entities.User`
                The update's creator
            creator_id : `str`
                The unique identifier of the update creator.
            id : `str`
                The update's unique identifier.
            item_id : `str`
                The update's item ID.
            replies : `list[moncli.entities.Reply]
                The update's replies.
            text_body : `str`
                The update's text body.
            updated_at : `str`
                The update's last edit date.
        
         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
    """
    
    kwargs['id'] = gql.IntValue(id)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=DELETE_UPDATE, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def create_notification(text: str, user_id: str, target_id: str, target_type: NotificationTargetType, *args, **kwargs):
    """Create a new notfication.

        Parameters
        
            text : `str`
                The notification text.
            user_id : `str`
                The user's unique identifier.
            target_id : `str`
                The target's unique identifier.
            target_type : `moncli.enums.NotificationTargetType`
                The target's type (Project / Post).
            args : `tuple`
                The list of notification return fields.
            kwargs : `dict`
                Optional arguments for creating notifications.

        Returns
            
            data : `dict`
                A monday.com notification in dictionary form.

        Return Fields
        
            id : `str`
                The notification's unique identifier.
            text : `str`
                The notification text.

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.        
            payload : `json`
                The notification payload.
    """
    
    kwargs['text'] = gql.StringValue(text)
    kwargs['user_id'] = gql.IntValue(user_id)
    kwargs['target_id'] = gql.IntValue(target_id)
    kwargs['target_type'] = gql.EnumValue(target_type)    
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=CREATE_NOTIFICATION, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def create_or_get_tag(tag_name: str, *args, **kwargs):
    """Create a new tag or get it if it already exists.

        Parameters
        
            tag_name : `str`
                The new tag's name.
            args : `tuple`
                The list of tag return fields.
            kwargs : `dict`
                Optional arguments for tag.

        Returns
            
            data : `dict`
                A monday.com tag in dictionary form.

        Return Fields
        
            color : `str`
                The tag's color.
            id : `str`
                The tag's unique identifier.
            name : `str`
                The tag's name.

        
         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.        
            board_id : `str`
                The private board id to create the tag at (not needed for public boards).
    """
    
    kwargs['tag_name'] = gql.StringValue(tag_name)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=CREATE_OR_GET_TAG, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def get_tags(*args, **kwargs):
    """Get a collection of tags.

        Parameters
        
            args : `tuple`
                The list of tag return fields.
            kwargs : `dict`
                Optional arguments for tag.

        Returns
            
            data : `dict`
                A monday.com tag in dictionary form.

        Return Fields
        
            color : `str`
                The tag's color.
            id : `str`
                The tag's unique identifier.
            name : `str`
                The tag's name.
        
         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
            ids : `list[str]`
                A list of tags unique identifiers.
    """
    
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=TAGS, operation_type=gql.OperationType.QUERY, fields=args, arguments=kwargs)


def add_file_to_update(update_id: str, file_path: str, *args, **kwargs):
    """Add a file to an update.

        Parameters
        
            update_id : `str`
                The update to add the file to.
            file_path : `str`
                The path of the file to upload.
            args : `tuple`
                The list of asset return fields.
            kwargs : `dict`
                Optional arguments for adding file to update.

        Returns
            
            data : `dict`
                A monday.com asset in dictionary form.

        Return Fields
        
            created_at : `str`
                The file's creation date.
            file_extension : `str`
                The file's extension.
            file_size : `int`
                The file's size in bytes.
            id : `str`
                The file's unique identifier.
            name : `str`
                The file's name.
            public_url : `str`
                Public url to the asset, valid for 1 hour.
            uploaded_by : `moncli.entities.user.User`
                The user who uploaded the file
            url : `str`
                The user who uploaded the file
            url_thumbnail : `str`
                Url to view the asset in thumbnail mode. Only available for images.  
        
         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
    """
    
    kwargs['file'] = gql.FileValue('$file')
    kwargs['update_id'] = gql.IntValue(update_id)
    return upload_file(file_path, api_key=kwargs.pop('api_key', None), query_name=ADD_FILE_TO_UPDATE, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def add_file_to_column(item_id: str, column_id: str, file_path: str, *args, **kwargs):
    """Add a file to a column value.

        Parameters
        
            item_id : `str`
                The item to add the file to.
            column_id : `str`
                The column to add the file to.
            file_path : `str`
                The path of the file to add.
            args : `tuple`
                The list of asset return fields.
            kwargs : `dict`
                Optional arguments for adding file to column.

        Returns
            
            data : `dict`
                A monday.com asset in dictionary form.

        Return Fields
        
            created_at : `str`
                The file's creation date.
            file_extension : `str`
                The file's extension.
            file_size : `int`
                The file's size in bytes.
            id : `str`
                The file's unique identifier.
            name : `str`
                The file's name.
            public_url : `str`
                Public url to the asset, valid for 1 hour.
            uploaded_by : `moncli.entities.user.User`
                The user who uploaded the file
            url : `str`
                The user who uploaded the file
            url_thumbnail : `str`
                Url to view the asset in thumbnail mode. Only available for images.

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
    """
    
    kwargs['file'] = gql.FileValue('$file')
    kwargs['item_id'] = gql.IntValue(item_id)
    kwargs['column_id'] = gql.StringValue(column_id)
    return upload_file(file_path, api_key=kwargs.pop('api_key', None), query_name=ADD_FILE_TO_COLUMN, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def get_users(*args, **kwargs):
    """Get a collection of users.

        Parameters
        
            args : `tuple`
                The list of user return fields.
            kwargs : `dict`
                Optional arguments for querying users.

        Returns
            
            data : `dict`
                A monday.com user in dictionary form.

        Return Fields
        
            account : `moncli.entities.Account`
                The user's account.
            birthday : `str`
                The user's birthday.
            country_code : `str`
                The user's country code.
            created_at : `str`
                The user's creation date.
            email : `str`
                The user's email.
            enabled : `bool`
                Is the user enabled or not.
            id : `str`
                The user's unique identifier.
            is_admin: `bool`
                Is the user a admin or not.
            is_guest : `bool`
                Is the user a guest or not.
            is_pending : `bool`
                Is the user a pending user.
            is_verified: `bool`
                Is the user is verified
            is_view_only : `bool`
                Is the user a view only user or not.
            join_date : `str`
                The date the user joined the account.
            location : `str`
                The user' location.
            mobile_phone : `str`
                The user's mobile phone number.
            name : `str`
                The user's name.
            phone : `str`
                The user's phone number.
            photo_original : `str`
                The user's photo in the original size.
            photo_small : `str`
                The user's photo in small size (150x150).
            photo_thumb : `str`
                The user's photo in thumbnail size (100x100).
            photo_thumb_small : `str`
                The user's photo in small thumbnail size (50x50).
            photo_tiny : `str`
                The user's photo in tiny size (30x30).
            teams : `list[moncli.entities.Team]`
                The teams the user is a member in.
            time_zone_identifier : `str`
                The user's time zone identifier.
            title : `str`
                The user's title.
            url : `str`
                The user's profile url.
            utc_hours_diff : `int`
                The user's UTC hours difference.

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
            ids : `list[str]`
                A list of users unique identifiers.
            kind : `moncli.enums.UserKind`
                The kind to search users by (all / non_guests / guests / non_pending).
            newest_first : `bool`
                Get the recently created users at the top of the list.
            limit : `int`
                Nimber of users to get.
    """
    
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=USERS, operation_type=gql.OperationType.QUERY, fields=args, arguments=kwargs)


def get_teams(*args, **kwargs):
    """Get a collection of teams.

        Parameters
        
            args : `tuple`
                The list of team return fields.
            kwargs : `dict`
                Optional arguments for querying teams.

        Returns
            
            data : `dict`
                A monday.com team in dictionary form.

        Return Fields
        
            id : `int`
                The team's unique identifier.
            name : `str`
                The team's name.
            picture_url : `str`
                The team's picture url.
            users : `moncli.entities.user.User`
                The users in the team.

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
            ids : `list[str]`
                A list of teams unique identifiers.
    """
    
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=TEAMS, operation_type=gql.OperationType.QUERY, fields=args, arguments=kwargs)


def get_me(*args, **kwargs):
    """Get the connected user's information.

        Parameters
        
            args : `tuple`
                The list of user return fields.
            kwargs : `dict`
                Optional arguments for getting my user.

        Returns
            
            data : `dict`
                A monday.com user in dictionary form.

        Return Fields
        
            account : `moncli.entities.Account`
                The user's account.
            birthday : `str`
                The user's birthday.
            country_code : `str`
                The user's country code.
            created_at : `str`
                The user's creation date.
            email : `str`
                The user's email.
            enabled : `bool`
                Is the user enabled or not.
            id : `str`
                The user's unique identifier.
            is_admin: `bool`
                Is the user a admin or not.
            is_guest : `bool`
                Is the user a guest or not.
            is_pending : `bool`
                Is the user a pending user.
            is_verified: `bool`
                Is the user is verified
            is_view_only : `bool`
                Is the user a view only user or not.
            join_date : `str`
                The date the user joined the account.
            location : `str`
                The user' location.
            mobile_phone : `str`
                The user's mobile phone number.
            name : `str`
                The user's name.
            phone : `str`
                The user's phone number.
            photo_original : `str`
                The user's photo in the original size.
            photo_small : `str`
                The user's photo in small size (150x150).
            photo_thumb : `str`
                The user's photo in thumbnail size (100x100).
            photo_thumb_small : `str`
                The user's photo in small thumbnail size (50x50).
            photo_tiny : `str`
                The user's photo in tiny size (30x30).
            teams : `list[moncli.entities.Team]`
                The teams the user is a member in.
            time_zone_identifier : `str`
                The user's time zone identifier.
            title : `str`
                The user's title.
            url : `str`
                The user's profile url.
            utc_hours_diff : `int`
                The user's UTC hours difference.

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
    """
    
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=ME, operation_type=gql.OperationType.QUERY, fields=args, arguments=kwargs)


def get_account(*args, **kwargs):
    """Get the connected account's information.

        Parameters
        
            api_key : `str`
                The monday.com v2 API user key.
            args : `tuple`
                The list of account return fields.
            kwargs : `dict`
                Optional arguments for querying accounts.

        Returns
            
            data : `dict`
                A monday.com account in dictionary form.

        Return Fields
        
            first_day_of_the_week : `str`
                The first day of the week for the account (sunday / monday).
            id : `int`
                The account's unique identifier.
            logo : `str`
                The account's logo.
            name : `str`
                The account's name.
            plan : `moncli.entities.Plan`
                The account's payment plan.
            show_timeline_weekends : `bool`
                Show weekends in timeline.
            slug : `str`
                The account's slug.

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
    """
    
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=ACCOUNT, operation_type=gql.OperationType.QUERY, fields=args, arguments=kwargs)


def create_webhook(board_id: str, url: str, event: WebhookEventType, *args, **kwargs):
    """Create a new webhook.

        Parameters
        
            api_key : `str`
                The monday.com v2 API user key.
            board_id : `str`
                The board's unique identifier.
            url : `str`
                The webhook URL.
            event : `moncli.enum.WebhookEventType`
                The event to listen to (incoming_notification / change_column_value / change_specific_column_value / create_item / create_update).
            args : `tuple`
                The list of webhook return fields.
            kwargs : `dict`
                Optional arguments for creating webhook.

        Returns
            
            data : `dict`
                A monday.com webhook in dictionary form.

        Return Fields
        
            board_id : `str`
                The webhook's board id.
            id : `str`
                The webhook's unique identifier.

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.        
            config : `json`
                The webhook config.
    """
    
    kwargs['board_id'] = gql.IntValue(board_id)
    kwargs['url'] = gql.StringValue(url)
    kwargs['event'] = gql.EnumValue(event)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=CREATE_WEBHOOK, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def delete_webhook(webhook_id: str, *args, **kwargs):
    """Delete a new webhook.

        Parameters
        
            webhook_id : `str`
                The webhook's unique identifier.
            args : `tuple`
                The list of webhook return fields.
            kwargs : `dict`
                Optional arguments for deleting webhook.

        Returns
            
            data : `dict`
                A monday.com webhook in dictionary form.

        Return Fields
        
            board_id : `str`
                The webhook's board id.
            id : `str`
                The webhook's unique identifier.

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.
    """
    
    kwargs['id'] = gql.IntValue(webhook_id)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=DELETE_WEBHOOK, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def create_workspace(name: str, kind: WorkspaceKind, *args, **kwargs):
    """Create a new workspace.

        Parameters
        
            name : `str`
                The Workspace's name
            kind : `moncli.enums.WorkspaceKind`
                The workspace's kind (open / closed)
            args : `tuple`
                The list of workspace return fields.
            kwargs : `dict`
                Optional arguments for creating workspace.

        Returns
            
            data : `dict`
                A monday.com workspace in dictionary form.

        Return Fields
        
            id : `str`
                The workspace's unique identifier.
            name : `str`
                The workspace's name.
            kind : `str`
                The workspace's kind (open / closed)
            description : `str`
                The workspace's description

         Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.        
            description : `str`
                The Workspace's description.
    """
    
    kwargs['name'] = gql.StringValue(name)
    kwargs['kind'] = gql.EnumValue(kind)
    return execute_query(api_key=kwargs.pop('api_key', None), query_name=CREATE_WORKSPACE, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def add_users_to_workspace(workspace_id: str,user_ids: list(), kind: WorkspaceSubscriberKind, *args, **kwargs):
        """Allows you to add users to a workspace. You can define if users will be added as regular subscribers or as owners of the workspace.

            Parameters 

                workspace_id: int
                     The workspace's unique identifier
                user_ids: `list[str]`
                    User IDs to subscribe to the workspace
                kind: WorkspaceSUbscriberKind
                    Kind of subscribers added (subscriber/owner)
                *args: tuple
                    The collection of workspace return fields.
            
            Return Fields

                account : `moncli.entities.Account`
                    The user's account.
                birthday : `str`
                    The user's birthday.
                country_code : `str`
                    The user's country code.
                created_at : `str`
                    The user's creation date.
                email : `str`
                    The user's email.
                enabled : `bool`
                    Is the user enabled or not.
                id : `str`
                    The user's unique identifier.
                is_admin: `bool`
                    Is the user a admin or not.   
                is_guest : `bool`
                    Is the user a guest or not.
                is_pending : `bool`
                    Is the user a pending user.
                is_view_only : `bool`
                    Is the user a view only user or not.
                join_date : `str`
                    The date the user joined the account.
                location : `str`
                    The user' location.
                mobile_phone : `str`
                    The user's mobile phone number.
                name : `str`
                    The user's name.
                phone : `str`
                    The user's phone number.
                photo_original : `str`
                    The user's photo in the original size.
                photo_small : `str`
                    The user's photo in small size (150x150).
                photo_thumb : `str`
                    The user's photo in thumbnail size (100x100).
                photo_thumb_small : `str`
                    The user's photo in small thumbnail size (50x50).
                photo_tiny : `str`
                    The user's photo in tiny size (30x30).
                teams : `list[moncli.entities.Team]`
                    The teams the user is a member in.
                time_zone_identifier : `str`
                    The user's time zone identifier.
                title : `str`
                    The user's title.
                url : `str`
                    The user's profile url.
                utc_hours_diff : `int`
                    The user's UTC hours difference.
            
            Optional Arguments

                api_key: `str`
                    The monday.com v2 API user key.
                kwargs: dict
                    Additional Arguments
        """

        kwargs["workspace_id"]= gql.IntValue(workspace_id)      
        kwargs['user_ids'] = gql.ListValue([int(id) for id in user_ids])
        kwargs["kind"]= gql.EnumValue(kind)
        
        return execute_query(api_key=kwargs.pop('api_key', None), query_name=ADD_USERS_TO_WORKSPACE, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)

      
def delete_users_from_workspace(workspace_id: str, user_ids: list(), *args: tuple, **kwargs: dict ):
    """Allows you to delete users to a workspace.

            Parameters 

                workspace_id: int
                        The workspace's unique identifier
                user_ids: `list[str]`
                        User IDs to subscribe to the workspace
                *args: tuple
                    The collection of workspace return fields.
            
            Return Fields
                
                account : `moncli.entities.Account`
                    The user's account.
                birthday : `str`
                    The user's birthday.
                country_code : `str`
                    The user's country code.
                created_at : `str`
                    The user's creation date.
                email : `str`
                    The user's email.
                enabled : `bool`
                    Is the user enabled or not.
                id : `str`
                    The user's unique identifier.
                is_admin: `bool`
                    Is the user a admin or not.
                is_guest : `bool`
                    Is the user a guest or not.
                is_pending : `bool`
                    Is the user a pending user.
                is_view_only : `bool`
                    Is the user a view only user or not.
                join_date : `str`
                    The date the user joined the account.
                location : `str`
                    The user' location.
                mobile_phone : `str`
                    The user's mobile phone number.
                name : `str`
                    The user's name.
                phone : `str`
                    The user's phone number.
                photo_original : `str`
                    The user's photo in the original size.
                photo_small : `str`
                    The user's photo in small size (150x150).
                photo_thumb : `str`
                    The user's photo in thumbnail size (100x100).
                photo_thumb_small : `str`
                    The user's photo in small thumbnail size (50x50).
                photo_tiny : `str`
                    The user's photo in tiny size (30x30).
                teams : `list[moncli.entities.Team]`
                    The teams the user is a member in.
                time_zone_identifier : `str`
                    The user's time zone identifier.
                title : `str`
                    The user's title.
                url : `str`
                    The user's profile url.
                utc_hours_diff : `int`
                    The user's UTC hours difference.
                
            Optional Arguments

                api_key: `str`
                    The monday.com v2 API user key.
                kwargs: dict
                    Additional Arguments
                
    """
    
    kwargs["workspace_id"]= gql.IntValue(workspace_id)
    kwargs['user_ids'] = gql.ListValue([int(id) for id in user_ids])

    return execute_query(api_key=kwargs.pop('api_key', None), query_name=DELETE_USERS_FROM_WORKSPACE, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)

  
def add_teams_to_workspace(workspace_id: str, team_ids: list(), *args, **kwargs ):
    """
        Allows you to add teams to a workspace.

        Parameters

            workspace_id: str
                The workspace's unique identifier.
            team_ids: list()
                Team ids to subscribe to the workspace.
        
        Returns
            
            data : `dict`
                A monday.com team in dictionary form.

        Return Fields
        
            id : `int`
                The team's unique identifier.
            name : `str`
                The team's name.
            picture_url : `str`
                The team's picture url.
            users : `moncli.entities.user.User`
                The users in the team.

    """
    kwargs['workspace_id'] = gql.IntValue(workspace_id)
    kwargs['team_ids'] = gql.ListValue(int(id) for id in team_ids)

    return execute_query(api_key=kwargs.pop('api_key', None), query_name=ADD_TEAMS_TO_WORKSPACE, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


def delete_teams_from_workspace(workspace_id: str, team_ids: list(), *args, **kwargs ):
    """
        Allows you to remove teams to a workspace.

        Parameters

            workspace_id: str
                The workspace's unique identifier.
            team_ids: list()
                Team ids to subscribe to the workspace.
                
        Returns
            
            data : `dict`
                A monday.com team in dictionary form.

        Return Fields
        
            id : `int`
                The team's unique identifier.
            name : `str`
                The team's name.
            picture_url : `str`
                The team's picture url.
            users : `moncli.entities.user.User`
                The users in the team. 

        
        Optional Arguments

            api_key : `str`
                The monday.com v2 API user key.


    """
    kwargs['workspace_id'] = gql.IntValue(workspace_id)
    kwargs['team_ids'] = gql.ListValue(int(id) for id in team_ids)

    return execute_query(api_key=kwargs.pop('api_key', None), query_name=DELETE_TEAMS_FROM_WORKSPACE, operation_type=gql.OperationType.MUTATION, fields=args, arguments=kwargs)


from moncli.routes import MondayQueryParameters, format_url, raise_mondayapi_error
from .. import constants
import requests, json


def execute_get(api_key, resource_url, params = None):

    monday_params = MondayQueryParameters(api_key)

    if params != None:
        monday_params.add_params(params)

    resp = requests.get(
        format_url(resource_url), 
        params=monday_params.to_dict(),
        timeout=constants.TIMEOUT)

    if resp.status_code == 200:
        return resp.json()

    raise_mondayapi_error('GET', resource_url, resp)


def execute_post(api_key, resource_url, body):

    monday_params = MondayQueryParameters(api_key)

    resp = requests.post(
        format_url(resource_url),
        body,
        params=monday_params.to_dict(),
        timeout=constants.TIMEOUT)

    if resp.status_code == 200 or resp.status_code == 201:
        return resp.json()

    raise_mondayapi_error('POST', resource_url, resp)


def execute_put(api_key, resource_url, body):

    monday_params = MondayQueryParameters(api_key)

    data = json.dumps(body)

    resp = requests.post(
        format_url(resource_url),
        data,
        params=monday_params.to_dict(),
        timeout=constants.TIMEOUT)

    if resp.status_code == 200:
        return resp.json()

    raise_mondayapi_error('PUT', resource_url, resp)


def execute_delete(api_key, resource_url, params = None):

    monday_params = MondayQueryParameters(api_key)

    if params != None:
        monday_params.add_params(params)

    resp = requests.delete(
        format_url(resource_url), 
        params=monday_params.to_dict(),
        timeout=constants.TIMEOUT)

    if resp.status_code == 200:
        return resp.json()

    raise_mondayapi_error('GET', resource_url, resp)


from moncli.routes import constants
from moncli.routes.requests import execute_get

def get_tag_by_id(api_key, tag_id):

    resource_url = constants.TAGS_BY_ID.format(tag_id)

    return execute_get(api_key, resource_url)


from moncli.api_v1 import constants
from moncli.api_v1.requests import execute_get

def get_users(api_key, page = 1, per_page = 25, offset = 0, order_by_latest = False):

    resource_url = constants.USERS

    params = {
        'page': page,
        'per_page': per_page,
        'offset': offset,
        'order_by_latest': order_by_latest
    }

    return execute_get(api_key, resource_url, params)


def get_user_by_id(api_key, user_id):

    resource_url = constants.USER_BY_ID.format(user_id)

    return execute_get(api_key, resource_url)


def get_user_posts(api_key, user_id, page = 1, per_page = 25, offset = 0):

    resource_url = constants.USER_POSTS.format(user_id)

    params = {
        'page': page,
        'per_page': per_page,
        'offset': offset
    }

    return execute_get(api_key, resource_url, params)


def get_user_newsfeed(api_key, user_id, page = 1, per_page = 25, offset = 0):

    resource_url = constants.USER_NEWSFEED.format(user_id)

    params = {
        'page': page,
        'per_page': per_page,
        'offset': offset
    }

    return execute_get(api_key, resource_url, params)


def get_user_unread_feed(api_key, user_id, page = 1, per_page = 25, offset = 0):

    resource_url = constants.USER_UNREAD_FEED.format(user_id)

    params = {
        'page': page,
        'per_page': per_page,
        'offset': offset
    }

    return execute_get(api_key, resource_url, params)

